# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Starsme2(KaitaiStruct):
    """:field s: s
    :field beacon_no: beacon_no
    :field rssi: rssi
    :field micro_switch_status_deployment: micro_switch_status_deployment
    :field micro_switch_status_mission: micro_switch_status_mission
    :field micro_switch_status_paddle: micro_switch_status_paddle
    :field bus_voltage: bus_voltage
    :field bus_current: bus_current
    :field main_cpu_voltage: main_cpu_voltage
    :field battery_voltage: battery_voltage
    :field beacon: beacon
    
    .. seealso::
       Source - http://stars.eng.shizuoka.ac.jp/english/CW_Telemetry_format.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.identifier = (self._io.read_bytes(6)).decode(u"ASCII")
        if not  ((self.identifier == u"jj2yxo")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.identifier, self._io, u"/seq/0")
        self.s = (self._io.read_bytes(1)).decode(u"ASCII")
        if not  ((self.s == u"s")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.s, self._io, u"/seq/1")
        self.beacon_no = (self._io.read_bytes(1)).decode(u"ASCII")
        if not  ((self.beacon_no == u"1") or (self.beacon_no == u"2")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.beacon_no, self._io, u"/seq/2")
        self.value1 = self._io.read_u1()
        self.value2 = self._io.read_u1()
        self.value3 = self._io.read_u1()
        self.value4 = self._io.read_u1()
        self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

    @property
    def value3_hex_right_digit(self):
        if hasattr(self, '_m_value3_hex_right_digit'):
            return self._m_value3_hex_right_digit

        self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
        return getattr(self, '_m_value3_hex_right_digit', None)

    @property
    def micro_switch_status_deployment(self):
        if hasattr(self, '_m_micro_switch_status_deployment'):
            return self._m_micro_switch_status_deployment

        if self.beacon_no == u"1":
            self._m_micro_switch_status_deployment = self.value2

        return getattr(self, '_m_micro_switch_status_deployment', None)

    @property
    def necessary_for_lengthcheck(self):
        if hasattr(self, '_m_necessary_for_lengthcheck'):
            return self._m_necessary_for_lengthcheck

        if len(self.lengthcheck) != 0:
            self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

        return getattr(self, '_m_necessary_for_lengthcheck', None)

    @property
    def value1_hex_left(self):
        if hasattr(self, '_m_value1_hex_left'):
            return self._m_value1_hex_left

        self._m_value1_hex_left = self.value1 // 16
        return getattr(self, '_m_value1_hex_left', None)

    @property
    def micro_switch_status_mission(self):
        if hasattr(self, '_m_micro_switch_status_mission'):
            return self._m_micro_switch_status_mission

        if self.beacon_no == u"1":
            self._m_micro_switch_status_mission = self.value3

        return getattr(self, '_m_micro_switch_status_mission', None)

    @property
    def value2_hex_right(self):
        if hasattr(self, '_m_value2_hex_right'):
            return self._m_value2_hex_right

        self._m_value2_hex_right = (self.value2 % 16)
        return getattr(self, '_m_value2_hex_right', None)

    @property
    def value3_hex(self):
        if hasattr(self, '_m_value3_hex'):
            return self._m_value3_hex

        self._m_value3_hex = self.value3_hex_left_digit + self.value3_hex_right_digit
        return getattr(self, '_m_value3_hex', None)

    @property
    def rssi(self):
        if hasattr(self, '_m_rssi'):
            return self._m_rssi

        if self.beacon_no == u"1":
            self._m_rssi = self.value1

        return getattr(self, '_m_rssi', None)

    @property
    def value3_hex_right(self):
        if hasattr(self, '_m_value3_hex_right'):
            return self._m_value3_hex_right

        self._m_value3_hex_right = (self.value3 % 16)
        return getattr(self, '_m_value3_hex_right', None)

    @property
    def battery_voltage(self):
        if hasattr(self, '_m_battery_voltage'):
            return self._m_battery_voltage

        if self.beacon_no == u"2":
            self._m_battery_voltage = (self.value4 * 0.0195)

        return getattr(self, '_m_battery_voltage', None)

    @property
    def value1_hex_right_digit(self):
        if hasattr(self, '_m_value1_hex_right_digit'):
            return self._m_value1_hex_right_digit

        self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
        return getattr(self, '_m_value1_hex_right_digit', None)

    @property
    def beacon(self):
        if hasattr(self, '_m_beacon'):
            return self._m_beacon

        self._m_beacon = self.s + self.beacon_no + u" " + self.value1_hex + u" " + self.value2_hex + u" " + self.value3_hex + u" " + self.value4_hex
        return getattr(self, '_m_beacon', None)

    @property
    def value3_hex_left(self):
        if hasattr(self, '_m_value3_hex_left'):
            return self._m_value3_hex_left

        self._m_value3_hex_left = self.value3 // 16
        return getattr(self, '_m_value3_hex_left', None)

    @property
    def value4_hex_left_digit(self):
        if hasattr(self, '_m_value4_hex_left_digit'):
            return self._m_value4_hex_left_digit

        self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
        return getattr(self, '_m_value4_hex_left_digit', None)

    @property
    def value1_hex_right(self):
        if hasattr(self, '_m_value1_hex_right'):
            return self._m_value1_hex_right

        self._m_value1_hex_right = (self.value1 % 16)
        return getattr(self, '_m_value1_hex_right', None)

    @property
    def value4_hex_right(self):
        if hasattr(self, '_m_value4_hex_right'):
            return self._m_value4_hex_right

        self._m_value4_hex_right = (self.value4 % 16)
        return getattr(self, '_m_value4_hex_right', None)

    @property
    def value4_hex_right_digit(self):
        if hasattr(self, '_m_value4_hex_right_digit'):
            return self._m_value4_hex_right_digit

        self._m_value4_hex_right_digit = (u"a" if str(self.value4_hex_right) == u"10" else (u"b" if str(self.value4_hex_right) == u"11" else (u"c" if str(self.value4_hex_right) == u"12" else (u"d" if str(self.value4_hex_right) == u"13" else (u"e" if str(self.value4_hex_right) == u"14" else (u"f" if str(self.value4_hex_right) == u"15" else str(self.value4_hex_right)))))))
        return getattr(self, '_m_value4_hex_right_digit', None)

    @property
    def value1_hex(self):
        if hasattr(self, '_m_value1_hex'):
            return self._m_value1_hex

        self._m_value1_hex = self.value1_hex_left_digit + self.value1_hex_right_digit
        return getattr(self, '_m_value1_hex', None)

    @property
    def value2_hex_right_digit(self):
        if hasattr(self, '_m_value2_hex_right_digit'):
            return self._m_value2_hex_right_digit

        self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
        return getattr(self, '_m_value2_hex_right_digit', None)

    @property
    def value4_hex_left(self):
        if hasattr(self, '_m_value4_hex_left'):
            return self._m_value4_hex_left

        self._m_value4_hex_left = self.value4 // 16
        return getattr(self, '_m_value4_hex_left', None)

    @property
    def value2_hex_left_digit(self):
        if hasattr(self, '_m_value2_hex_left_digit'):
            return self._m_value2_hex_left_digit

        self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
        return getattr(self, '_m_value2_hex_left_digit', None)

    @property
    def main_cpu_voltage(self):
        if hasattr(self, '_m_main_cpu_voltage'):
            return self._m_main_cpu_voltage

        if self.beacon_no == u"2":
            self._m_main_cpu_voltage = (self.value3 * 0.039)

        return getattr(self, '_m_main_cpu_voltage', None)

    @property
    def value2_hex(self):
        if hasattr(self, '_m_value2_hex'):
            return self._m_value2_hex

        self._m_value2_hex = self.value2_hex_left_digit + self.value2_hex_right_digit
        return getattr(self, '_m_value2_hex', None)

    @property
    def value4_hex(self):
        if hasattr(self, '_m_value4_hex'):
            return self._m_value4_hex

        self._m_value4_hex = self.value4_hex_left_digit + self.value4_hex_right_digit
        return getattr(self, '_m_value4_hex', None)

    @property
    def bus_current(self):
        if hasattr(self, '_m_bus_current'):
            return self._m_bus_current

        if self.beacon_no == u"2":
            self._m_bus_current = (self.value2 * 0.0296)

        return getattr(self, '_m_bus_current', None)

    @property
    def value3_hex_left_digit(self):
        if hasattr(self, '_m_value3_hex_left_digit'):
            return self._m_value3_hex_left_digit

        self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
        return getattr(self, '_m_value3_hex_left_digit', None)

    @property
    def bus_voltage(self):
        if hasattr(self, '_m_bus_voltage'):
            return self._m_bus_voltage

        if self.beacon_no == u"2":
            self._m_bus_voltage = (self.value1 * 0.039)

        return getattr(self, '_m_bus_voltage', None)

    @property
    def micro_switch_status_paddle(self):
        if hasattr(self, '_m_micro_switch_status_paddle'):
            return self._m_micro_switch_status_paddle

        if self.beacon_no == u"1":
            self._m_micro_switch_status_paddle = self.value4

        return getattr(self, '_m_micro_switch_status_paddle', None)

    @property
    def value2_hex_left(self):
        if hasattr(self, '_m_value2_hex_left'):
            return self._m_value2_hex_left

        self._m_value2_hex_left = self.value2 // 16
        return getattr(self, '_m_value2_hex_left', None)

    @property
    def value1_hex_left_digit(self):
        if hasattr(self, '_m_value1_hex_left_digit'):
            return self._m_value1_hex_left_digit

        self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
        return getattr(self, '_m_value1_hex_left_digit', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Geoscan16u(KaitaiStruct):
    """:field callsign_start_str: data.ax25_header.dest_callsign_raw.callsign_ror.callsign_start.callsign_start_str
    :field callsign_end_str: data.ax25_header.dest_callsign_raw.callsign_ror.callsign_end.callsign_end_str
    :field ssid_mask: data.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: data.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign_start_str: data.ax25_header.src_callsign_raw.callsign_ror.callsign_start.callsign_start_str
    :field src_callsign_raw_callsign_end_str: data.ax25_header.src_callsign_raw.callsign_ror.callsign_end.callsign_end_str
    :field src_ssid_raw_ssid_mask: data.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: data.ax25_header.src_ssid_raw.ssid
    :field ctl: data.ax25_header.ctl
    :field pid: data.ax25_header.pid
    :field beacon_id: data.payload.beacon_id
    :field eps_1_mode: data.payload.eps_1_mode
    :field eps_1_consumption_current: data.payload.eps_1_consumption_current
    :field eps_1_solar_cells_current: data.payload.eps_1_solar_cells_current
    :field eps_1_cell_voltage_full: data.payload.eps_1_cell_voltage_full
    :field eps_1_battery_temperature: data.payload.eps_1_battery_temperature
    :field eps_1_temperature_sp_y_pos: data.payload.eps_1_temperature_sp_y_pos
    :field eps_1_temperature_sp_y_neg: data.payload.eps_1_temperature_sp_y_neg
    :field eps_1_temperature_sp_x_pos: data.payload.eps_1_temperature_sp_x_pos
    :field eps_1_temperature_sp_x_neg: data.payload.eps_1_temperature_sp_x_neg
    :field eps_1_systems_status: data.payload.eps_1_systems_status
    :field eps_1_boot_count: data.payload.eps_1_boot_count
    :field eps_2_mode: data.payload.eps_2_mode
    :field eps_2_consumption_current: data.payload.eps_2_consumption_current
    :field eps_2_solar_cells_current: data.payload.eps_2_solar_cells_current
    :field eps_2_cell_voltage_full: data.payload.eps_2_cell_voltage_full
    :field eps_2_battery_temperature: data.payload.eps_2_battery_temperature
    :field eps_2_temperature_sp_y_pos: data.payload.eps_2_temperature_sp_y_pos
    :field eps_2_temperature_sp_y_neg: data.payload.eps_2_temperature_sp_y_neg
    :field eps_2_temperature_sp_x_pos: data.payload.eps_2_temperature_sp_x_pos
    :field eps_2_temperature_sp_x_neg: data.payload.eps_2_temperature_sp_x_neg
    :field eps_2_systems_status: data.payload.eps_2_systems_status
    :field eps_2_boot_count: data.payload.eps_2_boot_count
    :field adcs_mt_mode: data.payload.adcs_mt_mode
    :field adcs_rm_mode: data.payload.adcs_rm_mode
    :field adcs_kf_mode: data.payload.adcs_kf_mode
    :field adcs_filter_reset_count: data.payload.adcs_filter_reset_count
    :field adcs_sensors_state: data.payload.adcs_sensors_state
    :field adcs_flywheel_state: data.payload.adcs_flywheel_state
    :field comm_type: data.payload.comm_type
    :field comm_vbus_voltage: data.payload.comm_vbus_voltage
    :field comm_boot_count: data.payload.comm_boot_count
    :field comm_rssi: data.payload.comm_rssi
    :field comm_rssi_minimal: data.payload.comm_rssi_minimal
    :field comm_received_valid_packets: data.payload.comm_received_valid_packets
    :field comm_received_invalid_packets: data.payload.comm_received_invalid_packets
    :field comm_sent_packets: data.payload.comm_sent_packets
    :field comm_status: data.payload.comm_status
    :field comm_mode: data.payload.comm_mode
    :field comm_amp_temperature: data.payload.comm_amp_temperature
    :field comm_reserved_1: data.payload.comm_reserved_1
    :field comm_reserved_2: data.payload.comm_reserved_2
    :field eps_1_mcu1ch1_obc: data.payload.eps_1_mcu1ch1_obc
    :field eps_1_mcu1ch2_commu1: data.payload.eps_1_mcu1ch2_commu1
    :field eps_1_mcu2ch1_comms_plasgu: data.payload.eps_1_mcu2ch1_comms_plasgu
    :field eps_1_mcu2ch2_commu2: data.payload.eps_1_mcu2ch2_commu2
    :field eps_1_mcu3ch1_adcs1: data.payload.eps_1_mcu3ch1_adcs1
    :field eps_1_mcu3ch2_starsns1: data.payload.eps_1_mcu3ch2_starsns1
    :field eps_1_mcu4ch1_payload3_lepton_bridge: data.payload.eps_1_mcu4ch1_payload3_lepton_bridge
    :field eps_1_mcu4ch2_commx: data.payload.eps_1_mcu4ch2_commx
    :field eps_1_mcu5ch1_gyro1: data.payload.eps_1_mcu5ch1_gyro1
    :field eps_1_mcu6ch1_adcs2: data.payload.eps_1_mcu6ch1_adcs2
    :field eps_1_mcu6ch2_starsns2: data.payload.eps_1_mcu6ch2_starsns2
    :field eps_1_mcu7ch1_rws: data.payload.eps_1_mcu7ch1_rws
    :field eps_1_mcu7ch2_fecu_rtr: data.payload.eps_1_mcu7ch2_fecu_rtr
    :field eps_1_mcu8ch1_payload1_heater: data.payload.eps_1_mcu8ch1_payload1_heater
    :field eps_1_mcu8ch2_payload2_camera_heater: data.payload.eps_1_mcu8ch2_payload2_camera_heater
    :field eps_1_timekeeper: data.payload.eps_1_timekeeper
    :field eps_2_mcu1ch1_obc: data.payload.eps_2_mcu1ch1_obc
    :field eps_2_mcu1ch2_commu1: data.payload.eps_2_mcu1ch2_commu1
    :field eps_2_mcu2ch1_comms_plasgu: data.payload.eps_2_mcu2ch1_comms_plasgu
    :field eps_2_mcu2ch2_commu2: data.payload.eps_2_mcu2ch2_commu2
    :field eps_2_mcu3ch1_adcs1: data.payload.eps_2_mcu3ch1_adcs1
    :field eps_2_mcu3ch2_starsns1: data.payload.eps_2_mcu3ch2_starsns1
    :field eps_2_mcu4ch1_payload3_lepton_bridge: data.payload.eps_2_mcu4ch1_payload3_lepton_bridge
    :field eps_2_mcu4ch2_commx: data.payload.eps_2_mcu4ch2_commx
    :field eps_2_mcu5ch1_gyro1: data.payload.eps_2_mcu5ch1_gyro1
    :field eps_2_mcu6ch1_adcs2: data.payload.eps_2_mcu6ch1_adcs2
    :field eps_2_mcu6ch2_starsns2: data.payload.eps_2_mcu6ch2_starsns2
    :field eps_2_mcu7ch1_rws: data.payload.eps_2_mcu7ch1_rws
    :field eps_2_mcu7ch2_fecu_rtr: data.payload.eps_2_mcu7ch2_fecu_rtr
    :field eps_2_mcu8ch1_payload1_heater: data.payload.eps_2_mcu8ch1_payload1_heater
    :field eps_2_mcu8ch2_payload2_camera_heater: data.payload.eps_2_mcu8ch2_payload2_camera_heater
    :field eps_2_timekeeper: data.payload.eps_2_timekeeper
    :field comm_antenna_detector: data.payload.comm_antenna_detector
    :field comm_mode_correct: data.payload.comm_mode_correct
    :field comm_fec_on: data.payload.comm_fec_on
    :field packet_id: data.data_header.packet_id
    :field packet_size: data.data_header.packet_size
    :field system: data.data_payload.system
    :field gnss_type_byte: data.data_payload.payload.gnss_type_byte
    :field posix_time_ms: data.data_payload.payload.gnss_payload.posix_time_ms
    :field coord_x: data.data_payload.payload.gnss_payload.coord_x
    :field coord_y: data.data_payload.payload.gnss_payload.coord_y
    :field coord_z: data.data_payload.payload.gnss_payload.coord_z
    :field velocity_x: data.data_payload.payload.gnss_payload.velocity_x
    :field velocity_y: data.data_payload.payload.gnss_payload.velocity_y
    :field velocity_z: data.data_payload.payload.gnss_payload.velocity_z
    :field quaternion_q0: data.data_payload.payload.gnss_payload.quaternion_q0
    :field quaternion_q1: data.data_payload.payload.gnss_payload.quaternion_q1
    :field quaternion_q2: data.data_payload.payload.gnss_payload.quaternion_q2
    :field quaternion_q3: data.data_payload.payload.gnss_payload.quaternion_q3
    :field ref_motion_mode: data.data_payload.payload.gnss_payload.ref_motion_mode
    :field mag_torquer_mode: data.data_payload.payload.gnss_payload.mag_torquer_mode
    :field filter_type: data.data_payload.payload.gnss_payload.filter_type
    :field orientation_system_status: data.data_payload.payload.gnss_payload.orientation_system_status
    :field reserved: data.data_payload.payload.gnss_payload.reserved
    :field azdk_quaternion_q1: data.data_payload.payload.gnss_payload.azdk_quaternion_q1
    :field azdk_quaternion_q2: data.data_payload.payload.gnss_payload.azdk_quaternion_q2
    :field azdk_quaternion_q3: data.data_payload.payload.gnss_payload.azdk_quaternion_q3
    :field posix_time_s: data.data_payload.payload.gnss_payload.posix_time_s
    :field angular_velocity_x: data.data_payload.payload.gnss_payload.angular_velocity_x
    :field angular_velocity_y: data.data_payload.payload.gnss_payload.angular_velocity_y
    :field angular_velocity_z: data.data_payload.payload.gnss_payload.angular_velocity_z
    :field reserved0: data.data_payload.payload.gnss_payload.reserved0
    :field reserved1: data.data_payload.payload.gnss_payload.reserved1
    :field num: data.data_payload.payload.gnss_payload.num
    :field posix_time_s_1: data.data_payload.payload.gnss_payload.posix_time_s_1
    :field coord_x_1: data.data_payload.payload.gnss_payload.coord_x_1
    :field coord_y_1: data.data_payload.payload.gnss_payload.coord_y_1
    :field coord_z_1: data.data_payload.payload.gnss_payload.coord_z_1
    :field velocity_x_1: data.data_payload.payload.gnss_payload.velocity_x_1
    :field velocity_y_1: data.data_payload.payload.gnss_payload.velocity_y_1
    :field velocity_z_1: data.data_payload.payload.gnss_payload.velocity_z_1
    :field posix_time_s_2: data.data_payload.payload.gnss_payload.posix_time_s_2
    :field coord_x_2: data.data_payload.payload.gnss_payload.coord_x_2
    :field coord_y_2: data.data_payload.payload.gnss_payload.coord_y_2
    :field coord_z_2: data.data_payload.payload.gnss_payload.coord_z_2
    :field velocity_x_2: data.data_payload.payload.gnss_payload.velocity_x_2
    :field velocity_y_2: data.data_payload.payload.gnss_payload.velocity_y_2
    :field velocity_z_2: data.data_payload.payload.gnss_payload.velocity_z_2
    :field eps_beacon_base_info_raw: data.data_payload.payload.eps_beacon_base_info_raw
    :field eps_beacon_reboot_count: data.data_payload.payload.eps_beacon_reboot_count
    :field eps_beacon_firmware_image: data.data_payload.payload.eps_beacon_firmware_image
    :field eps_beacon_switcher_info_raw: data.data_payload.payload.eps_beacon_switcher_info_raw
    :field eps_beacon_balancer_info_raw: data.data_payload.payload.eps_beacon_balancer_info_raw
    :field eps_beacon_deployer_info_raw: data.data_payload.payload.eps_beacon_deployer_info_raw
    :field eps_beacon_heater_info_raw_1: data.data_payload.payload.eps_beacon_heater_info_raw_1
    :field eps_beacon_heater_info_raw_2: data.data_payload.payload.eps_beacon_heater_info_raw_2
    :field eps_beacon_error_mask_raw_1: data.data_payload.payload.eps_beacon_error_mask_raw_1
    :field eps_beacon_error_mask_raw_2: data.data_payload.payload.eps_beacon_error_mask_raw_2
    :field eps_beacon_load_current: data.data_payload.payload.eps_beacon_load_current
    :field eps_beacon_solar_current: data.data_payload.payload.eps_beacon_solar_current
    :field eps_beacon_cell1_voltage: data.data_payload.payload.eps_beacon_cell1_voltage
    :field eps_beacon_cell2_voltage: data.data_payload.payload.eps_beacon_cell2_voltage
    :field eps_beacon_cell3_voltage: data.data_payload.payload.eps_beacon_cell3_voltage
    :field eps_beacon_coulomb_counter: data.data_payload.payload.eps_beacon_coulomb_counter
    :field eps_beacon_mask_enabled_ext_devices_raw: data.data_payload.payload.eps_beacon_mask_enabled_ext_devices_raw
    :field eps_beacon_mask_enabled_int_devices_raw: data.data_payload.payload.eps_beacon_mask_enabled_int_devices_raw
    :field eps_beacon_temp_eps_battery: data.data_payload.payload.eps_beacon_temp_eps_battery
    :field eps_beacon_temp_eps_main: data.data_payload.payload.eps_beacon_temp_eps_main
    :field eps_beacon_temp_commu: data.data_payload.payload.eps_beacon_temp_commu
    :field eps_beacon_temp_commx: data.data_payload.payload.eps_beacon_temp_commx
    :field eps_beacon_temp_plasgu: data.data_payload.payload.eps_beacon_temp_plasgu
    :field eps_beacon_temp_rws0: data.data_payload.payload.eps_beacon_temp_rws0
    :field eps_beacon_temp_rws1: data.data_payload.payload.eps_beacon_temp_rws1
    :field eps_beacon_temp_rws2: data.data_payload.payload.eps_beacon_temp_rws2
    :field eps_beacon_temp_rws3: data.data_payload.payload.eps_beacon_temp_rws3
    :field eps_beacon_temp_solar_wing_y_pos: data.data_payload.payload.eps_beacon_temp_solar_wing_y_pos
    :field eps_beacon_temp_solar_wing_y_neg: data.data_payload.payload.eps_beacon_temp_solar_wing_y_neg
    :field eps_beacon_temp_camera_internal: data.data_payload.payload.eps_beacon_temp_camera_internal
    :field eps_beacon_temp_camera_heater_plate: data.data_payload.payload.eps_beacon_temp_camera_heater_plate
    :field eps_beacon_base_info_eps_mod: data.data_payload.payload.eps_beacon_base_info_eps_mod
    :field eps_beacon_base_info_self_addr: data.data_payload.payload.eps_beacon_base_info_self_addr
    :field eps_beacon_switcher_current_commu: data.data_payload.payload.eps_beacon_switcher_current_commu
    :field eps_beacon_switcher_state: data.data_payload.payload.eps_beacon_switcher_state
    :field eps_beacon_switcher_switching_state: data.data_payload.payload.eps_beacon_switcher_switching_state
    :field eps_beacon_switcher_relay_switch_error: data.data_payload.payload.eps_beacon_switcher_relay_switch_error
    :field eps_beacon_switcher_read_data_error: data.data_payload.payload.eps_beacon_switcher_read_data_error
    :field eps_beacon_balancer_state: data.data_payload.payload.eps_beacon_balancer_state
    :field eps_beacon_balancer_error: data.data_payload.payload.eps_beacon_balancer_error
    :field eps_beacon_balancer_is_balancing: data.data_payload.payload.eps_beacon_balancer_is_balancing
    :field eps_beacon_balancer_active_balancer: data.data_payload.payload.eps_beacon_balancer_active_balancer
    :field eps_beacon_balancer_ind_balancing_cell: data.data_payload.payload.eps_beacon_balancer_ind_balancing_cell
    :field eps_beacon_deployer_state: data.data_payload.payload.eps_beacon_deployer_state
    :field eps_beacon_deployer_storage_error: data.data_payload.payload.eps_beacon_deployer_storage_error
    :field eps_beacon_deployer_deployed_channels_mask: data.data_payload.payload.eps_beacon_deployer_deployed_channels_mask
    :field eps_beacon_heater_mode: data.data_payload.payload.eps_beacon_heater_mode
    :field eps_beacon_heater_active_heater_mask: data.data_payload.payload.eps_beacon_heater_active_heater_mask
    :field eps_beacon_heater_mask_tmp_src: data.data_payload.payload.eps_beacon_heater_mask_tmp_src
    :field eps_beacon_error_i2c_bat_monitor: data.data_payload.payload.eps_beacon_error_i2c_bat_monitor
    :field eps_beacon_error_spi_bat_monitor: data.data_payload.payload.eps_beacon_error_spi_bat_monitor
    :field eps_beacon_error_spi_fram: data.data_payload.payload.eps_beacon_error_spi_fram
    :field eps_beacon_error_i2c_fram1: data.data_payload.payload.eps_beacon_error_i2c_fram1
    :field eps_beacon_error_i2c_fram2: data.data_payload.payload.eps_beacon_error_i2c_fram2
    :field eps_beacon_error_mcu1_i2c: data.data_payload.payload.eps_beacon_error_mcu1_i2c
    :field eps_beacon_error_mcu2_i2c: data.data_payload.payload.eps_beacon_error_mcu2_i2c
    :field eps_beacon_error_mcu3_i2c: data.data_payload.payload.eps_beacon_error_mcu3_i2c
    :field eps_beacon_error_mcu4_i2c: data.data_payload.payload.eps_beacon_error_mcu4_i2c
    :field eps_beacon_error_mcu5_i2c: data.data_payload.payload.eps_beacon_error_mcu5_i2c
    :field eps_beacon_error_mcu6_i2c: data.data_payload.payload.eps_beacon_error_mcu6_i2c
    :field eps_beacon_error_mcu7_i2c: data.data_payload.payload.eps_beacon_error_mcu7_i2c
    :field eps_beacon_error_mcu8_i2c: data.data_payload.payload.eps_beacon_error_mcu8_i2c
    :field eps_beacon_ext_mcu1ch1: data.data_payload.payload.eps_beacon_ext_mcu1ch1
    :field eps_beacon_ext_mcu1ch2: data.data_payload.payload.eps_beacon_ext_mcu1ch2
    :field eps_beacon_ext_mcu2ch1: data.data_payload.payload.eps_beacon_ext_mcu2ch1
    :field eps_beacon_ext_mcu2ch2: data.data_payload.payload.eps_beacon_ext_mcu2ch2
    :field eps_beacon_ext_mcu3ch1: data.data_payload.payload.eps_beacon_ext_mcu3ch1
    :field eps_beacon_ext_mcu3ch2: data.data_payload.payload.eps_beacon_ext_mcu3ch2
    :field eps_beacon_ext_mcu4ch1: data.data_payload.payload.eps_beacon_ext_mcu4ch1
    :field eps_beacon_ext_mcu4ch2: data.data_payload.payload.eps_beacon_ext_mcu4ch2
    :field eps_beacon_ext_mcu5ch1: data.data_payload.payload.eps_beacon_ext_mcu5ch1
    :field eps_beacon_ext_mcu6ch1: data.data_payload.payload.eps_beacon_ext_mcu6ch1
    :field eps_beacon_ext_mcu6ch2: data.data_payload.payload.eps_beacon_ext_mcu6ch2
    :field eps_beacon_ext_mcu7ch1: data.data_payload.payload.eps_beacon_ext_mcu7ch1
    :field eps_beacon_ext_mcu7ch2: data.data_payload.payload.eps_beacon_ext_mcu7ch2
    :field eps_beacon_ext_mcu8ch1: data.data_payload.payload.eps_beacon_ext_mcu8ch1
    :field eps_beacon_ext_mcu8ch2: data.data_payload.payload.eps_beacon_ext_mcu8ch2
    :field eps_beacon_ext_timekeeper: data.data_payload.payload.eps_beacon_ext_timekeeper
    :field eps_beacon_int_mcu_pwr1: data.data_payload.payload.eps_beacon_int_mcu_pwr1
    :field eps_beacon_int_mcu_pwr2: data.data_payload.payload.eps_beacon_int_mcu_pwr2
    :field eps_beacon_int_ds_bus1: data.data_payload.payload.eps_beacon_int_ds_bus1
    :field eps_beacon_int_ds_bus2: data.data_payload.payload.eps_beacon_int_ds_bus2
    :field eps_beacon_int_can1: data.data_payload.payload.eps_beacon_int_can1
    :field eps_beacon_int_can2: data.data_payload.payload.eps_beacon_int_can2
    :field eps_beacon_int_timebus: data.data_payload.payload.eps_beacon_int_timebus
    :field eps_beacon_int_buxbus: data.data_payload.payload.eps_beacon_int_buxbus
    :field eps_beacon_int_fram1: data.data_payload.payload.eps_beacon_int_fram1
    :field eps_beacon_int_fram2: data.data_payload.payload.eps_beacon_int_fram2
    :field eps_beacon_int_bat_balancer1: data.data_payload.payload.eps_beacon_int_bat_balancer1
    :field eps_beacon_int_bat_balancer2: data.data_payload.payload.eps_beacon_int_bat_balancer2
    :field len: len
    :field type: type
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        _on = self.type
        if _on == 33930:
            self.data = Geoscan16u.Ax25Frame(self._io, self, self._root)
        else:
            self.data = Geoscan16u.DataFrame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Geoscan16u.Ax25Header(self._io, self, self._root)
            self.payload = Geoscan16u.GeoscanBeaconTlm(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Geoscan16u.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Geoscan16u.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Geoscan16u.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Geoscan16u.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign_start = Geoscan16u.CallsignStartRaw(self._io, self, self._root)
            self.callsign_end = Geoscan16u.CallsignEndRaw(self._io, self, self._root)


    class GnssTlm70(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.num = self._io.read_u4le()
            self.posix_time_s_1 = self._io.read_u4le()
            self.coord_x_1 = self._io.read_s4le()
            self.coord_y_1 = self._io.read_s4le()
            self.coord_z_1 = self._io.read_s4le()
            self.velocity_x_1 = self._io.read_s4le()
            self.velocity_y_1 = self._io.read_s4le()
            self.velocity_z_1 = self._io.read_s4le()
            self.posix_time_s_2 = self._io.read_u4le()
            self.coord_x_2 = self._io.read_s4le()
            self.coord_y_2 = self._io.read_s4le()
            self.coord_z_2 = self._io.read_s4le()
            self.velocity_x_2 = self._io.read_s4le()
            self.velocity_y_2 = self._io.read_s4le()
            self.velocity_z_2 = self._io.read_s4le()
            self.reserved0 = self._io.read_u4le()
            self.reserved1 = self._io.read_u2le()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class DataHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_id = self._io.read_u2le()
            self.packet_size = self._io.read_u1()


    class CallsignStartRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign_start_str = (self._io.read_bytes(2)).decode(u"ASCII")
            if not  ((self.callsign_start_str == u"BE") or (self.callsign_start_str == u"RS")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign_start_str, self._io, u"/types/callsign_start_raw/seq/0")


    class DataTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.system = self._io.read_u2le()
            if not  ((self.system == 48437) or (self.system == 48425)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.system, self._io, u"/types/data_tlm/seq/0")
            _on = self.system
            if _on == 48437:
                self.payload = Geoscan16u.GnssTlm(self._io, self, self._root)
            elif _on == 48425:
                self.payload = Geoscan16u.EpsBeaconTlm(self._io, self, self._root)


    class EpsBeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_beacon_base_info_raw = self._io.read_u1()
            self.eps_beacon_reboot_count = self._io.read_u2le()
            self.eps_beacon_firmware_image = self._io.read_u1()
            self.eps_beacon_switcher_info_raw = self._io.read_u1()
            self.eps_beacon_balancer_info_raw = self._io.read_u1()
            self.eps_beacon_deployer_info_raw = self._io.read_u1()
            self.eps_beacon_heater_info_raw_1 = self._io.read_u1()
            self.eps_beacon_heater_info_raw_2 = self._io.read_u1()
            self.eps_beacon_error_mask_raw_1 = self._io.read_u1()
            self.eps_beacon_error_mask_raw_2 = self._io.read_u1()
            self.eps_beacon_load_current = self._io.read_u2le()
            self.eps_beacon_solar_current = self._io.read_u2le()
            self.eps_beacon_cell1_voltage = self._io.read_u2le()
            self.eps_beacon_cell2_voltage = self._io.read_u2le()
            self.eps_beacon_cell3_voltage = self._io.read_u2le()
            self.eps_beacon_coulomb_counter = self._io.read_s4le()
            self.eps_beacon_mask_enabled_ext_devices_raw = self._io.read_u2le()
            self.eps_beacon_mask_enabled_int_devices_raw = self._io.read_u2le()
            self.eps_beacon_temp_eps_battery = self._io.read_s1()
            self.eps_beacon_temp_eps_main = self._io.read_s1()
            self.eps_beacon_temp_commu = self._io.read_s1()
            self.eps_beacon_temp_commx = self._io.read_s1()
            self.eps_beacon_temp_plasgu = self._io.read_s1()
            self.eps_beacon_temp_rws0 = self._io.read_s1()
            self.eps_beacon_temp_rws1 = self._io.read_s1()
            self.eps_beacon_temp_rws2 = self._io.read_s1()
            self.eps_beacon_temp_rws3 = self._io.read_s1()
            self.eps_beacon_temp_solar_wing_y_pos = self._io.read_s1()
            self.eps_beacon_temp_solar_wing_y_neg = self._io.read_s1()
            self.eps_beacon_temp_camera_internal = self._io.read_s1()
            self.eps_beacon_temp_camera_heater_plate = self._io.read_s1()

        @property
        def eps_beacon_ext_mcu2ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu2ch2'):
                return self._m_eps_beacon_ext_mcu2ch2

            self._m_eps_beacon_ext_mcu2ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 3) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu2ch2', None)

        @property
        def eps_beacon_switcher_switching_state(self):
            if hasattr(self, '_m_eps_beacon_switcher_switching_state'):
                return self._m_eps_beacon_switcher_switching_state

            self._m_eps_beacon_switcher_switching_state = ((self.eps_beacon_switcher_info_raw >> 3) & 3)
            return getattr(self, '_m_eps_beacon_switcher_switching_state', None)

        @property
        def eps_beacon_base_info_self_addr(self):
            if hasattr(self, '_m_eps_beacon_base_info_self_addr'):
                return self._m_eps_beacon_base_info_self_addr

            self._m_eps_beacon_base_info_self_addr = ((self.eps_beacon_base_info_raw >> 3) & 31)
            return getattr(self, '_m_eps_beacon_base_info_self_addr', None)

        @property
        def eps_beacon_ext_mcu4ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu4ch1'):
                return self._m_eps_beacon_ext_mcu4ch1

            self._m_eps_beacon_ext_mcu4ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 6) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu4ch1', None)

        @property
        def eps_beacon_ext_mcu1ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu1ch1'):
                return self._m_eps_beacon_ext_mcu1ch1

            self._m_eps_beacon_ext_mcu1ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 0) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu1ch1', None)

        @property
        def eps_beacon_error_mcu1_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu1_i2c'):
                return self._m_eps_beacon_error_mcu1_i2c

            self._m_eps_beacon_error_mcu1_i2c = ((self.eps_beacon_error_mask_raw_1 >> 5) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu1_i2c', None)

        @property
        def eps_beacon_int_bat_balancer1(self):
            if hasattr(self, '_m_eps_beacon_int_bat_balancer1'):
                return self._m_eps_beacon_int_bat_balancer1

            self._m_eps_beacon_int_bat_balancer1 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 10) & 1)
            return getattr(self, '_m_eps_beacon_int_bat_balancer1', None)

        @property
        def eps_beacon_ext_mcu6ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu6ch2'):
                return self._m_eps_beacon_ext_mcu6ch2

            self._m_eps_beacon_ext_mcu6ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 10) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu6ch2', None)

        @property
        def eps_beacon_heater_mode(self):
            if hasattr(self, '_m_eps_beacon_heater_mode'):
                return self._m_eps_beacon_heater_mode

            self._m_eps_beacon_heater_mode = ((self.eps_beacon_heater_info_raw_1 >> 0) & 3)
            return getattr(self, '_m_eps_beacon_heater_mode', None)

        @property
        def eps_beacon_balancer_error(self):
            if hasattr(self, '_m_eps_beacon_balancer_error'):
                return self._m_eps_beacon_balancer_error

            self._m_eps_beacon_balancer_error = ((self.eps_beacon_balancer_info_raw >> 2) & 1)
            return getattr(self, '_m_eps_beacon_balancer_error', None)

        @property
        def eps_beacon_heater_active_heater_mask(self):
            if hasattr(self, '_m_eps_beacon_heater_active_heater_mask'):
                return self._m_eps_beacon_heater_active_heater_mask

            self._m_eps_beacon_heater_active_heater_mask = ((self.eps_beacon_heater_info_raw_1 >> 2) & 3)
            return getattr(self, '_m_eps_beacon_heater_active_heater_mask', None)

        @property
        def eps_beacon_int_fram1(self):
            if hasattr(self, '_m_eps_beacon_int_fram1'):
                return self._m_eps_beacon_int_fram1

            self._m_eps_beacon_int_fram1 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 8) & 1)
            return getattr(self, '_m_eps_beacon_int_fram1', None)

        @property
        def eps_beacon_int_timebus(self):
            if hasattr(self, '_m_eps_beacon_int_timebus'):
                return self._m_eps_beacon_int_timebus

            self._m_eps_beacon_int_timebus = ((self.eps_beacon_mask_enabled_int_devices_raw >> 6) & 1)
            return getattr(self, '_m_eps_beacon_int_timebus', None)

        @property
        def eps_beacon_ext_mcu4ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu4ch2'):
                return self._m_eps_beacon_ext_mcu4ch2

            self._m_eps_beacon_ext_mcu4ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 7) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu4ch2', None)

        @property
        def eps_beacon_error_mcu7_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu7_i2c'):
                return self._m_eps_beacon_error_mcu7_i2c

            self._m_eps_beacon_error_mcu7_i2c = ((self.eps_beacon_error_mask_raw_2 >> 3) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu7_i2c', None)

        @property
        def eps_beacon_deployer_state(self):
            if hasattr(self, '_m_eps_beacon_deployer_state'):
                return self._m_eps_beacon_deployer_state

            self._m_eps_beacon_deployer_state = ((self.eps_beacon_deployer_info_raw >> 0) & 3)
            return getattr(self, '_m_eps_beacon_deployer_state', None)

        @property
        def eps_beacon_int_bat_balancer2(self):
            if hasattr(self, '_m_eps_beacon_int_bat_balancer2'):
                return self._m_eps_beacon_int_bat_balancer2

            self._m_eps_beacon_int_bat_balancer2 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 11) & 1)
            return getattr(self, '_m_eps_beacon_int_bat_balancer2', None)

        @property
        def eps_beacon_error_mcu3_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu3_i2c'):
                return self._m_eps_beacon_error_mcu3_i2c

            self._m_eps_beacon_error_mcu3_i2c = ((self.eps_beacon_error_mask_raw_1 >> 7) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu3_i2c', None)

        @property
        def eps_beacon_int_fram2(self):
            if hasattr(self, '_m_eps_beacon_int_fram2'):
                return self._m_eps_beacon_int_fram2

            self._m_eps_beacon_int_fram2 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 9) & 1)
            return getattr(self, '_m_eps_beacon_int_fram2', None)

        @property
        def eps_beacon_int_can1(self):
            if hasattr(self, '_m_eps_beacon_int_can1'):
                return self._m_eps_beacon_int_can1

            self._m_eps_beacon_int_can1 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 4) & 1)
            return getattr(self, '_m_eps_beacon_int_can1', None)

        @property
        def eps_beacon_deployer_storage_error(self):
            if hasattr(self, '_m_eps_beacon_deployer_storage_error'):
                return self._m_eps_beacon_deployer_storage_error

            self._m_eps_beacon_deployer_storage_error = ((self.eps_beacon_deployer_info_raw >> 2) & 1)
            return getattr(self, '_m_eps_beacon_deployer_storage_error', None)

        @property
        def eps_beacon_ext_mcu3ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu3ch2'):
                return self._m_eps_beacon_ext_mcu3ch2

            self._m_eps_beacon_ext_mcu3ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 5) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu3ch2', None)

        @property
        def eps_beacon_int_can2(self):
            if hasattr(self, '_m_eps_beacon_int_can2'):
                return self._m_eps_beacon_int_can2

            self._m_eps_beacon_int_can2 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 5) & 1)
            return getattr(self, '_m_eps_beacon_int_can2', None)

        @property
        def eps_beacon_error_mcu4_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu4_i2c'):
                return self._m_eps_beacon_error_mcu4_i2c

            self._m_eps_beacon_error_mcu4_i2c = ((self.eps_beacon_error_mask_raw_2 >> 0) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu4_i2c', None)

        @property
        def eps_beacon_error_i2c_fram2(self):
            if hasattr(self, '_m_eps_beacon_error_i2c_fram2'):
                return self._m_eps_beacon_error_i2c_fram2

            self._m_eps_beacon_error_i2c_fram2 = ((self.eps_beacon_error_mask_raw_1 >> 4) & 1)
            return getattr(self, '_m_eps_beacon_error_i2c_fram2', None)

        @property
        def eps_beacon_ext_mcu2ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu2ch1'):
                return self._m_eps_beacon_ext_mcu2ch1

            self._m_eps_beacon_ext_mcu2ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 2) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu2ch1', None)

        @property
        def eps_beacon_ext_mcu5ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu5ch1'):
                return self._m_eps_beacon_ext_mcu5ch1

            self._m_eps_beacon_ext_mcu5ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 8) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu5ch1', None)

        @property
        def eps_beacon_heater_mask_tmp_src(self):
            if hasattr(self, '_m_eps_beacon_heater_mask_tmp_src'):
                return self._m_eps_beacon_heater_mask_tmp_src

            self._m_eps_beacon_heater_mask_tmp_src = ((self.eps_beacon_heater_info_raw_1 >> 4) & 15)
            return getattr(self, '_m_eps_beacon_heater_mask_tmp_src', None)

        @property
        def eps_beacon_ext_mcu7ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu7ch2'):
                return self._m_eps_beacon_ext_mcu7ch2

            self._m_eps_beacon_ext_mcu7ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 12) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu7ch2', None)

        @property
        def eps_beacon_base_info_eps_mod(self):
            if hasattr(self, '_m_eps_beacon_base_info_eps_mod'):
                return self._m_eps_beacon_base_info_eps_mod

            self._m_eps_beacon_base_info_eps_mod = ((self.eps_beacon_base_info_raw >> 0) & 7)
            return getattr(self, '_m_eps_beacon_base_info_eps_mod', None)

        @property
        def eps_beacon_switcher_read_data_error(self):
            if hasattr(self, '_m_eps_beacon_switcher_read_data_error'):
                return self._m_eps_beacon_switcher_read_data_error

            self._m_eps_beacon_switcher_read_data_error = ((self.eps_beacon_switcher_info_raw >> 6) & 1)
            return getattr(self, '_m_eps_beacon_switcher_read_data_error', None)

        @property
        def eps_beacon_ext_timekeeper(self):
            if hasattr(self, '_m_eps_beacon_ext_timekeeper'):
                return self._m_eps_beacon_ext_timekeeper

            self._m_eps_beacon_ext_timekeeper = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 15) & 1)
            return getattr(self, '_m_eps_beacon_ext_timekeeper', None)

        @property
        def eps_beacon_switcher_current_commu(self):
            if hasattr(self, '_m_eps_beacon_switcher_current_commu'):
                return self._m_eps_beacon_switcher_current_commu

            self._m_eps_beacon_switcher_current_commu = ((self.eps_beacon_switcher_info_raw >> 0) & 1)
            return getattr(self, '_m_eps_beacon_switcher_current_commu', None)

        @property
        def eps_beacon_error_mcu2_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu2_i2c'):
                return self._m_eps_beacon_error_mcu2_i2c

            self._m_eps_beacon_error_mcu2_i2c = ((self.eps_beacon_error_mask_raw_1 >> 6) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu2_i2c', None)

        @property
        def eps_beacon_ext_mcu1ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu1ch2'):
                return self._m_eps_beacon_ext_mcu1ch2

            self._m_eps_beacon_ext_mcu1ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 1) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu1ch2', None)

        @property
        def eps_beacon_int_mcu_pwr2(self):
            if hasattr(self, '_m_eps_beacon_int_mcu_pwr2'):
                return self._m_eps_beacon_int_mcu_pwr2

            self._m_eps_beacon_int_mcu_pwr2 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 1) & 1)
            return getattr(self, '_m_eps_beacon_int_mcu_pwr2', None)

        @property
        def eps_beacon_int_ds_bus1(self):
            if hasattr(self, '_m_eps_beacon_int_ds_bus1'):
                return self._m_eps_beacon_int_ds_bus1

            self._m_eps_beacon_int_ds_bus1 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 2) & 1)
            return getattr(self, '_m_eps_beacon_int_ds_bus1', None)

        @property
        def eps_beacon_error_spi_fram(self):
            if hasattr(self, '_m_eps_beacon_error_spi_fram'):
                return self._m_eps_beacon_error_spi_fram

            self._m_eps_beacon_error_spi_fram = ((self.eps_beacon_error_mask_raw_1 >> 2) & 1)
            return getattr(self, '_m_eps_beacon_error_spi_fram', None)

        @property
        def eps_beacon_ext_mcu3ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu3ch1'):
                return self._m_eps_beacon_ext_mcu3ch1

            self._m_eps_beacon_ext_mcu3ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 4) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu3ch1', None)

        @property
        def eps_beacon_deployer_deployed_channels_mask(self):
            if hasattr(self, '_m_eps_beacon_deployer_deployed_channels_mask'):
                return self._m_eps_beacon_deployer_deployed_channels_mask

            self._m_eps_beacon_deployer_deployed_channels_mask = ((self.eps_beacon_deployer_info_raw >> 4) & 15)
            return getattr(self, '_m_eps_beacon_deployer_deployed_channels_mask', None)

        @property
        def eps_beacon_balancer_ind_balancing_cell(self):
            if hasattr(self, '_m_eps_beacon_balancer_ind_balancing_cell'):
                return self._m_eps_beacon_balancer_ind_balancing_cell

            self._m_eps_beacon_balancer_ind_balancing_cell = ((self.eps_beacon_balancer_info_raw >> 6) & 3)
            return getattr(self, '_m_eps_beacon_balancer_ind_balancing_cell', None)

        @property
        def eps_beacon_int_mcu_pwr1(self):
            if hasattr(self, '_m_eps_beacon_int_mcu_pwr1'):
                return self._m_eps_beacon_int_mcu_pwr1

            self._m_eps_beacon_int_mcu_pwr1 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 0) & 1)
            return getattr(self, '_m_eps_beacon_int_mcu_pwr1', None)

        @property
        def eps_beacon_switcher_relay_switch_error(self):
            if hasattr(self, '_m_eps_beacon_switcher_relay_switch_error'):
                return self._m_eps_beacon_switcher_relay_switch_error

            self._m_eps_beacon_switcher_relay_switch_error = ((self.eps_beacon_switcher_info_raw >> 5) & 1)
            return getattr(self, '_m_eps_beacon_switcher_relay_switch_error', None)

        @property
        def eps_beacon_ext_mcu8ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu8ch1'):
                return self._m_eps_beacon_ext_mcu8ch1

            self._m_eps_beacon_ext_mcu8ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 13) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu8ch1', None)

        @property
        def eps_beacon_int_buxbus(self):
            if hasattr(self, '_m_eps_beacon_int_buxbus'):
                return self._m_eps_beacon_int_buxbus

            self._m_eps_beacon_int_buxbus = ((self.eps_beacon_mask_enabled_int_devices_raw >> 7) & 1)
            return getattr(self, '_m_eps_beacon_int_buxbus', None)

        @property
        def eps_beacon_switcher_state(self):
            if hasattr(self, '_m_eps_beacon_switcher_state'):
                return self._m_eps_beacon_switcher_state

            self._m_eps_beacon_switcher_state = ((self.eps_beacon_switcher_info_raw >> 1) & 3)
            return getattr(self, '_m_eps_beacon_switcher_state', None)

        @property
        def eps_beacon_ext_mcu8ch2(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu8ch2'):
                return self._m_eps_beacon_ext_mcu8ch2

            self._m_eps_beacon_ext_mcu8ch2 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 14) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu8ch2', None)

        @property
        def eps_beacon_error_mcu6_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu6_i2c'):
                return self._m_eps_beacon_error_mcu6_i2c

            self._m_eps_beacon_error_mcu6_i2c = ((self.eps_beacon_error_mask_raw_2 >> 2) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu6_i2c', None)

        @property
        def eps_beacon_int_ds_bus2(self):
            if hasattr(self, '_m_eps_beacon_int_ds_bus2'):
                return self._m_eps_beacon_int_ds_bus2

            self._m_eps_beacon_int_ds_bus2 = ((self.eps_beacon_mask_enabled_int_devices_raw >> 3) & 1)
            return getattr(self, '_m_eps_beacon_int_ds_bus2', None)

        @property
        def eps_beacon_error_i2c_fram1(self):
            if hasattr(self, '_m_eps_beacon_error_i2c_fram1'):
                return self._m_eps_beacon_error_i2c_fram1

            self._m_eps_beacon_error_i2c_fram1 = ((self.eps_beacon_error_mask_raw_1 >> 3) & 1)
            return getattr(self, '_m_eps_beacon_error_i2c_fram1', None)

        @property
        def eps_beacon_error_mcu5_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu5_i2c'):
                return self._m_eps_beacon_error_mcu5_i2c

            self._m_eps_beacon_error_mcu5_i2c = ((self.eps_beacon_error_mask_raw_2 >> 1) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu5_i2c', None)

        @property
        def eps_beacon_balancer_is_balancing(self):
            if hasattr(self, '_m_eps_beacon_balancer_is_balancing'):
                return self._m_eps_beacon_balancer_is_balancing

            self._m_eps_beacon_balancer_is_balancing = ((self.eps_beacon_balancer_info_raw >> 3) & 1)
            return getattr(self, '_m_eps_beacon_balancer_is_balancing', None)

        @property
        def eps_beacon_error_i2c_bat_monitor(self):
            if hasattr(self, '_m_eps_beacon_error_i2c_bat_monitor'):
                return self._m_eps_beacon_error_i2c_bat_monitor

            self._m_eps_beacon_error_i2c_bat_monitor = ((self.eps_beacon_error_mask_raw_1 >> 0) & 1)
            return getattr(self, '_m_eps_beacon_error_i2c_bat_monitor', None)

        @property
        def eps_beacon_balancer_active_balancer(self):
            if hasattr(self, '_m_eps_beacon_balancer_active_balancer'):
                return self._m_eps_beacon_balancer_active_balancer

            self._m_eps_beacon_balancer_active_balancer = ((self.eps_beacon_balancer_info_raw >> 4) & 3)
            return getattr(self, '_m_eps_beacon_balancer_active_balancer', None)

        @property
        def eps_beacon_ext_mcu6ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu6ch1'):
                return self._m_eps_beacon_ext_mcu6ch1

            self._m_eps_beacon_ext_mcu6ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 9) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu6ch1', None)

        @property
        def eps_beacon_ext_mcu7ch1(self):
            if hasattr(self, '_m_eps_beacon_ext_mcu7ch1'):
                return self._m_eps_beacon_ext_mcu7ch1

            self._m_eps_beacon_ext_mcu7ch1 = ((self.eps_beacon_mask_enabled_ext_devices_raw >> 11) & 1)
            return getattr(self, '_m_eps_beacon_ext_mcu7ch1', None)

        @property
        def eps_beacon_balancer_state(self):
            if hasattr(self, '_m_eps_beacon_balancer_state'):
                return self._m_eps_beacon_balancer_state

            self._m_eps_beacon_balancer_state = ((self.eps_beacon_balancer_info_raw >> 0) & 3)
            return getattr(self, '_m_eps_beacon_balancer_state', None)

        @property
        def eps_beacon_error_spi_bat_monitor(self):
            if hasattr(self, '_m_eps_beacon_error_spi_bat_monitor'):
                return self._m_eps_beacon_error_spi_bat_monitor

            self._m_eps_beacon_error_spi_bat_monitor = ((self.eps_beacon_error_mask_raw_1 >> 1) & 1)
            return getattr(self, '_m_eps_beacon_error_spi_bat_monitor', None)

        @property
        def eps_beacon_error_mcu8_i2c(self):
            if hasattr(self, '_m_eps_beacon_error_mcu8_i2c'):
                return self._m_eps_beacon_error_mcu8_i2c

            self._m_eps_beacon_error_mcu8_i2c = ((self.eps_beacon_error_mask_raw_2 >> 4) & 1)
            return getattr(self, '_m_eps_beacon_error_mcu8_i2c', None)


    class GnssTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gnss_type_byte = self._io.read_u1()
            _on = self.gnss_type_byte
            if _on == 66:
                self.gnss_payload = Geoscan16u.GnssTlm66(self._io, self, self._root)
            elif _on == 67:
                self.gnss_payload = Geoscan16u.GnssTlm67(self._io, self, self._root)
            elif _on == 70:
                self.gnss_payload = Geoscan16u.GnssTlm70(self._io, self, self._root)


    class GnssTlm67(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.posix_time_s = self._io.read_u4le()
            self.coord_x = self._io.read_f4le()
            self.coord_y = self._io.read_f4le()
            self.coord_z = self._io.read_f4le()
            self.velocity_x = self._io.read_f4le()
            self.velocity_y = self._io.read_f4le()
            self.velocity_z = self._io.read_f4le()
            self.quaternion_q0 = self._io.read_f4le()
            self.quaternion_q1 = self._io.read_f4le()
            self.quaternion_q2 = self._io.read_f4le()
            self.quaternion_q3 = self._io.read_f4le()
            self.angular_velocity_x = self._io.read_f4le()
            self.angular_velocity_y = self._io.read_f4le()
            self.angular_velocity_z = self._io.read_f4le()
            self.reserved0 = self._io.read_u2le()
            self.reserved1 = self._io.read_u8le()


    class GnssTlm66(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.posix_time_ms = self._io.read_u8le()
            self.coord_x = self._io.read_f4le()
            self.coord_y = self._io.read_f4le()
            self.coord_z = self._io.read_f4le()
            self.velocity_x = self._io.read_f4le()
            self.velocity_y = self._io.read_f4le()
            self.velocity_z = self._io.read_f4le()
            self.quaternion_q0 = self._io.read_f4le()
            self.quaternion_q1 = self._io.read_f4le()
            self.quaternion_q2 = self._io.read_f4le()
            self.quaternion_q3 = self._io.read_f4le()
            self.ref_motion_mode = self._io.read_u1()
            self.mag_torquer_mode = self._io.read_u1()
            self.filter_type = self._io.read_u1()
            self.orientation_system_status = self._io.read_u1()
            self.reserved = self._io.read_u2le()
            self.azdk_quaternion_q1 = self._io.read_f4le()
            self.azdk_quaternion_q2 = self._io.read_f4le()
            self.azdk_quaternion_q3 = self._io.read_f4le()


    class GeoscanBeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_id = self._io.read_u1()
            self.eps_1_mode = self._io.read_u1()
            self.eps_1_consumption_current = self._io.read_u2le()
            self.eps_1_solar_cells_current = self._io.read_u2le()
            self.eps_1_cell_voltage_full = self._io.read_u2le()
            self.eps_1_battery_temperature = self._io.read_s1()
            self.eps_1_temperature_sp_y_pos = self._io.read_s1()
            self.eps_1_temperature_sp_y_neg = self._io.read_s1()
            self.eps_1_temperature_sp_x_pos = self._io.read_s1()
            self.eps_1_temperature_sp_x_neg = self._io.read_s1()
            self.eps_1_systems_status = self._io.read_u2le()
            self.eps_1_boot_count = self._io.read_u2le()
            self.eps_2_mode = self._io.read_u1()
            self.eps_2_consumption_current = self._io.read_u2le()
            self.eps_2_solar_cells_current = self._io.read_u2le()
            self.eps_2_cell_voltage_full = self._io.read_u2le()
            self.eps_2_battery_temperature = self._io.read_s1()
            self.eps_2_temperature_sp_y_pos = self._io.read_s1()
            self.eps_2_temperature_sp_y_neg = self._io.read_s1()
            self.eps_2_temperature_sp_x_pos = self._io.read_s1()
            self.eps_2_temperature_sp_x_neg = self._io.read_s1()
            self.eps_2_systems_status = self._io.read_u2le()
            self.eps_2_boot_count = self._io.read_u2le()
            self.adcs_mt_mode = self._io.read_u1()
            self.adcs_rm_mode = self._io.read_u1()
            self.adcs_kf_mode = self._io.read_u1()
            self.adcs_filter_reset_count = self._io.read_u1()
            self.adcs_sensors_state = self._io.read_u2le()
            self.adcs_flywheel_state = self._io.read_u1()
            self.comm_type = self._io.read_u1()
            self.comm_vbus_voltage = self._io.read_u2le()
            self.comm_boot_count = self._io.read_u2le()
            self.comm_rssi = self._io.read_s1()
            self.comm_rssi_minimal = self._io.read_s1()
            self.comm_received_valid_packets = self._io.read_u1()
            self.comm_received_invalid_packets = self._io.read_u1()
            self.comm_sent_packets = self._io.read_u1()
            self.comm_status = self._io.read_u1()
            self.comm_mode = self._io.read_u1()
            self.comm_amp_temperature = self._io.read_s1()
            self.comm_reserved_1 = self._io.read_u2le()
            self.comm_reserved_2 = self._io.read_u1()

        @property
        def eps_2_mcu7ch2_fecu_rtr(self):
            if hasattr(self, '_m_eps_2_mcu7ch2_fecu_rtr'):
                return self._m_eps_2_mcu7ch2_fecu_rtr

            self._m_eps_2_mcu7ch2_fecu_rtr = ((self.eps_2_systems_status >> 12) & 1)
            return getattr(self, '_m_eps_2_mcu7ch2_fecu_rtr', None)

        @property
        def eps_2_mcu8ch2_payload2_camera_heater(self):
            if hasattr(self, '_m_eps_2_mcu8ch2_payload2_camera_heater'):
                return self._m_eps_2_mcu8ch2_payload2_camera_heater

            self._m_eps_2_mcu8ch2_payload2_camera_heater = ((self.eps_2_systems_status >> 14) & 1)
            return getattr(self, '_m_eps_2_mcu8ch2_payload2_camera_heater', None)

        @property
        def eps_2_mcu1ch2_commu1(self):
            if hasattr(self, '_m_eps_2_mcu1ch2_commu1'):
                return self._m_eps_2_mcu1ch2_commu1

            self._m_eps_2_mcu1ch2_commu1 = ((self.eps_2_systems_status >> 1) & 1)
            return getattr(self, '_m_eps_2_mcu1ch2_commu1', None)

        @property
        def eps_1_mcu1ch2_commu1(self):
            if hasattr(self, '_m_eps_1_mcu1ch2_commu1'):
                return self._m_eps_1_mcu1ch2_commu1

            self._m_eps_1_mcu1ch2_commu1 = ((self.eps_1_systems_status >> 1) & 1)
            return getattr(self, '_m_eps_1_mcu1ch2_commu1', None)

        @property
        def eps_1_mcu6ch2_starsns2(self):
            if hasattr(self, '_m_eps_1_mcu6ch2_starsns2'):
                return self._m_eps_1_mcu6ch2_starsns2

            self._m_eps_1_mcu6ch2_starsns2 = ((self.eps_1_systems_status >> 10) & 1)
            return getattr(self, '_m_eps_1_mcu6ch2_starsns2', None)

        @property
        def comm_antenna_detector(self):
            if hasattr(self, '_m_comm_antenna_detector'):
                return self._m_comm_antenna_detector

            self._m_comm_antenna_detector = ((self.comm_status >> 0) & 1)
            return getattr(self, '_m_comm_antenna_detector', None)

        @property
        def eps_1_mcu4ch1_payload3_lepton_bridge(self):
            if hasattr(self, '_m_eps_1_mcu4ch1_payload3_lepton_bridge'):
                return self._m_eps_1_mcu4ch1_payload3_lepton_bridge

            self._m_eps_1_mcu4ch1_payload3_lepton_bridge = ((self.eps_1_systems_status >> 6) & 1)
            return getattr(self, '_m_eps_1_mcu4ch1_payload3_lepton_bridge', None)

        @property
        def eps_2_mcu6ch1_adcs2(self):
            if hasattr(self, '_m_eps_2_mcu6ch1_adcs2'):
                return self._m_eps_2_mcu6ch1_adcs2

            self._m_eps_2_mcu6ch1_adcs2 = ((self.eps_2_systems_status >> 9) & 1)
            return getattr(self, '_m_eps_2_mcu6ch1_adcs2', None)

        @property
        def eps_1_mcu7ch1_rws(self):
            if hasattr(self, '_m_eps_1_mcu7ch1_rws'):
                return self._m_eps_1_mcu7ch1_rws

            self._m_eps_1_mcu7ch1_rws = ((self.eps_1_systems_status >> 11) & 1)
            return getattr(self, '_m_eps_1_mcu7ch1_rws', None)

        @property
        def eps_1_mcu8ch1_payload1_heater(self):
            if hasattr(self, '_m_eps_1_mcu8ch1_payload1_heater'):
                return self._m_eps_1_mcu8ch1_payload1_heater

            self._m_eps_1_mcu8ch1_payload1_heater = ((self.eps_1_systems_status >> 13) & 1)
            return getattr(self, '_m_eps_1_mcu8ch1_payload1_heater', None)

        @property
        def eps_1_mcu5ch1_gyro1(self):
            if hasattr(self, '_m_eps_1_mcu5ch1_gyro1'):
                return self._m_eps_1_mcu5ch1_gyro1

            self._m_eps_1_mcu5ch1_gyro1 = ((self.eps_1_systems_status >> 8) & 1)
            return getattr(self, '_m_eps_1_mcu5ch1_gyro1', None)

        @property
        def eps_1_mcu1ch1_obc(self):
            if hasattr(self, '_m_eps_1_mcu1ch1_obc'):
                return self._m_eps_1_mcu1ch1_obc

            self._m_eps_1_mcu1ch1_obc = ((self.eps_1_systems_status >> 0) & 1)
            return getattr(self, '_m_eps_1_mcu1ch1_obc', None)

        @property
        def eps_2_timekeeper(self):
            if hasattr(self, '_m_eps_2_timekeeper'):
                return self._m_eps_2_timekeeper

            self._m_eps_2_timekeeper = ((self.eps_2_systems_status >> 15) & 1)
            return getattr(self, '_m_eps_2_timekeeper', None)

        @property
        def eps_2_mcu2ch1_comms_plasgu(self):
            if hasattr(self, '_m_eps_2_mcu2ch1_comms_plasgu'):
                return self._m_eps_2_mcu2ch1_comms_plasgu

            self._m_eps_2_mcu2ch1_comms_plasgu = ((self.eps_2_systems_status >> 2) & 1)
            return getattr(self, '_m_eps_2_mcu2ch1_comms_plasgu', None)

        @property
        def eps_2_mcu1ch1_obc(self):
            if hasattr(self, '_m_eps_2_mcu1ch1_obc'):
                return self._m_eps_2_mcu1ch1_obc

            self._m_eps_2_mcu1ch1_obc = ((self.eps_2_systems_status >> 0) & 1)
            return getattr(self, '_m_eps_2_mcu1ch1_obc', None)

        @property
        def eps_2_mcu3ch1_adcs1(self):
            if hasattr(self, '_m_eps_2_mcu3ch1_adcs1'):
                return self._m_eps_2_mcu3ch1_adcs1

            self._m_eps_2_mcu3ch1_adcs1 = ((self.eps_2_systems_status >> 4) & 1)
            return getattr(self, '_m_eps_2_mcu3ch1_adcs1', None)

        @property
        def eps_2_mcu6ch2_starsns2(self):
            if hasattr(self, '_m_eps_2_mcu6ch2_starsns2'):
                return self._m_eps_2_mcu6ch2_starsns2

            self._m_eps_2_mcu6ch2_starsns2 = ((self.eps_2_systems_status >> 10) & 1)
            return getattr(self, '_m_eps_2_mcu6ch2_starsns2', None)

        @property
        def eps_2_mcu8ch1_payload1_heater(self):
            if hasattr(self, '_m_eps_2_mcu8ch1_payload1_heater'):
                return self._m_eps_2_mcu8ch1_payload1_heater

            self._m_eps_2_mcu8ch1_payload1_heater = ((self.eps_2_systems_status >> 13) & 1)
            return getattr(self, '_m_eps_2_mcu8ch1_payload1_heater', None)

        @property
        def eps_2_mcu4ch2_commx(self):
            if hasattr(self, '_m_eps_2_mcu4ch2_commx'):
                return self._m_eps_2_mcu4ch2_commx

            self._m_eps_2_mcu4ch2_commx = ((self.eps_2_systems_status >> 7) & 1)
            return getattr(self, '_m_eps_2_mcu4ch2_commx', None)

        @property
        def comm_fec_on(self):
            if hasattr(self, '_m_comm_fec_on'):
                return self._m_comm_fec_on

            self._m_comm_fec_on = ((self.comm_status >> 2) & 1)
            return getattr(self, '_m_comm_fec_on', None)

        @property
        def eps_1_timekeeper(self):
            if hasattr(self, '_m_eps_1_timekeeper'):
                return self._m_eps_1_timekeeper

            self._m_eps_1_timekeeper = ((self.eps_1_systems_status >> 15) & 1)
            return getattr(self, '_m_eps_1_timekeeper', None)

        @property
        def eps_1_mcu3ch2_starsns1(self):
            if hasattr(self, '_m_eps_1_mcu3ch2_starsns1'):
                return self._m_eps_1_mcu3ch2_starsns1

            self._m_eps_1_mcu3ch2_starsns1 = ((self.eps_1_systems_status >> 5) & 1)
            return getattr(self, '_m_eps_1_mcu3ch2_starsns1', None)

        @property
        def eps_1_mcu4ch2_commx(self):
            if hasattr(self, '_m_eps_1_mcu4ch2_commx'):
                return self._m_eps_1_mcu4ch2_commx

            self._m_eps_1_mcu4ch2_commx = ((self.eps_1_systems_status >> 7) & 1)
            return getattr(self, '_m_eps_1_mcu4ch2_commx', None)

        @property
        def eps_2_mcu2ch2_commu2(self):
            if hasattr(self, '_m_eps_2_mcu2ch2_commu2'):
                return self._m_eps_2_mcu2ch2_commu2

            self._m_eps_2_mcu2ch2_commu2 = ((self.eps_2_systems_status >> 3) & 1)
            return getattr(self, '_m_eps_2_mcu2ch2_commu2', None)

        @property
        def eps_1_mcu2ch2_commu2(self):
            if hasattr(self, '_m_eps_1_mcu2ch2_commu2'):
                return self._m_eps_1_mcu2ch2_commu2

            self._m_eps_1_mcu2ch2_commu2 = ((self.eps_1_systems_status >> 3) & 1)
            return getattr(self, '_m_eps_1_mcu2ch2_commu2', None)

        @property
        def eps_1_mcu2ch1_comms_plasgu(self):
            if hasattr(self, '_m_eps_1_mcu2ch1_comms_plasgu'):
                return self._m_eps_1_mcu2ch1_comms_plasgu

            self._m_eps_1_mcu2ch1_comms_plasgu = ((self.eps_1_systems_status >> 2) & 1)
            return getattr(self, '_m_eps_1_mcu2ch1_comms_plasgu', None)

        @property
        def comm_mode_correct(self):
            if hasattr(self, '_m_comm_mode_correct'):
                return self._m_comm_mode_correct

            self._m_comm_mode_correct = ((self.comm_status >> 1) & 1)
            return getattr(self, '_m_comm_mode_correct', None)

        @property
        def eps_1_mcu3ch1_adcs1(self):
            if hasattr(self, '_m_eps_1_mcu3ch1_adcs1'):
                return self._m_eps_1_mcu3ch1_adcs1

            self._m_eps_1_mcu3ch1_adcs1 = ((self.eps_1_systems_status >> 4) & 1)
            return getattr(self, '_m_eps_1_mcu3ch1_adcs1', None)

        @property
        def eps_2_mcu7ch1_rws(self):
            if hasattr(self, '_m_eps_2_mcu7ch1_rws'):
                return self._m_eps_2_mcu7ch1_rws

            self._m_eps_2_mcu7ch1_rws = ((self.eps_2_systems_status >> 11) & 1)
            return getattr(self, '_m_eps_2_mcu7ch1_rws', None)

        @property
        def eps_2_mcu3ch2_starsns1(self):
            if hasattr(self, '_m_eps_2_mcu3ch2_starsns1'):
                return self._m_eps_2_mcu3ch2_starsns1

            self._m_eps_2_mcu3ch2_starsns1 = ((self.eps_2_systems_status >> 5) & 1)
            return getattr(self, '_m_eps_2_mcu3ch2_starsns1', None)

        @property
        def eps_1_mcu6ch1_adcs2(self):
            if hasattr(self, '_m_eps_1_mcu6ch1_adcs2'):
                return self._m_eps_1_mcu6ch1_adcs2

            self._m_eps_1_mcu6ch1_adcs2 = ((self.eps_1_systems_status >> 9) & 1)
            return getattr(self, '_m_eps_1_mcu6ch1_adcs2', None)

        @property
        def eps_2_mcu4ch1_payload3_lepton_bridge(self):
            if hasattr(self, '_m_eps_2_mcu4ch1_payload3_lepton_bridge'):
                return self._m_eps_2_mcu4ch1_payload3_lepton_bridge

            self._m_eps_2_mcu4ch1_payload3_lepton_bridge = ((self.eps_2_systems_status >> 6) & 1)
            return getattr(self, '_m_eps_2_mcu4ch1_payload3_lepton_bridge', None)

        @property
        def eps_2_mcu5ch1_gyro1(self):
            if hasattr(self, '_m_eps_2_mcu5ch1_gyro1'):
                return self._m_eps_2_mcu5ch1_gyro1

            self._m_eps_2_mcu5ch1_gyro1 = ((self.eps_2_systems_status >> 8) & 1)
            return getattr(self, '_m_eps_2_mcu5ch1_gyro1', None)

        @property
        def eps_1_mcu8ch2_payload2_camera_heater(self):
            if hasattr(self, '_m_eps_1_mcu8ch2_payload2_camera_heater'):
                return self._m_eps_1_mcu8ch2_payload2_camera_heater

            self._m_eps_1_mcu8ch2_payload2_camera_heater = ((self.eps_1_systems_status >> 14) & 1)
            return getattr(self, '_m_eps_1_mcu8ch2_payload2_camera_heater', None)

        @property
        def eps_1_mcu7ch2_fecu_rtr(self):
            if hasattr(self, '_m_eps_1_mcu7ch2_fecu_rtr'):
                return self._m_eps_1_mcu7ch2_fecu_rtr

            self._m_eps_1_mcu7ch2_fecu_rtr = ((self.eps_1_systems_status >> 12) & 1)
            return getattr(self, '_m_eps_1_mcu7ch2_fecu_rtr', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Geoscan16u.Callsign(_io__raw_callsign_ror, self, self._root)


    class CallsignEndRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign_end_str = (self._io.read_bytes(4)).decode(u"ASCII")


    class DataFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data_header = Geoscan16u.DataHeader(self._io, self, self._root)
            self.data_payload = Geoscan16u.DataTlm(self._io, self, self._root)


    @property
    def len(self):
        if hasattr(self, '_m_len'):
            return self._m_len

        self._m_len = self._io.size()
        return getattr(self, '_m_len', None)

    @property
    def type(self):
        if hasattr(self, '_m_type'):
            return self._m_type

        _pos = self._io.pos()
        self._io.seek(0)
        self._m_type = self._io.read_u2be()
        self._io.seek(_pos)
        return getattr(self, '_m_type', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Hadesicm(KaitaiStruct):
    """:field type: cubesat_frame.first_header.type
    :field address: cubesat_frame.first_header.address
    :field sclock: cubesat_frame.metadata.sclock
    :field peaksignal: cubesat_frame.metadata.peaksignal
    :field modasignal: cubesat_frame.metadata.modasignal
    :field lastcmdsignal: cubesat_frame.metadata.lastcmdsignal
    :field lastcmdnoise: cubesat_frame.metadata.lastcmdnoise
    :field spi: cubesat_frame.metadata.spi
    :field spa: cubesat_frame.metadata.spa
    :field spb: cubesat_frame.metadata.spb
    :field spc: cubesat_frame.metadata.spc
    :field spd: cubesat_frame.metadata.spd
    :field vbus1: cubesat_frame.metadata.vbus1
    :field vbus2: cubesat_frame.metadata.vbus2
    :field vbus3: cubesat_frame.metadata.vbus3
    :field vbat1: cubesat_frame.metadata.vbat1
    :field vbat2: cubesat_frame.metadata.vbat2
    :field vbus1_vbat1: cubesat_frame.metadata.vbus1_vbat1
    :field vbus3_vbus2: cubesat_frame.metadata.vbus3_vbus2
    :field vcpu: cubesat_frame.metadata.vcpu
    :field icpu: cubesat_frame.metadata.icpu
    :field ipl: cubesat_frame.metadata.ipl
    :field ibat: cubesat_frame.metadata.ibat
    :field tpa: cubesat_frame.metadata.tpa
    :field tpb: cubesat_frame.metadata.tpb
    :field tpc: cubesat_frame.metadata.tpc
    :field tpd: cubesat_frame.metadata.tpd
    :field ttx: cubesat_frame.metadata.ttx
    :field ttx2: cubesat_frame.metadata.ttx2
    :field trx: cubesat_frame.metadata.trx
    :field tcpu: cubesat_frame.metadata.tcpu
    :field uptime: cubesat_frame.metadata.uptime
    :field nrun: cubesat_frame.metadata.nrun
    :field npayload: cubesat_frame.metadata.npayload
    :field nwire: cubesat_frame.metadata.nwire
    :field ntransponder: cubesat_frame.metadata.ntransponder
    :field ntasksnotexecuted: cubesat_frame.metadata.ntasksnotexecuted
    :field antennadeployed: cubesat_frame.metadata.antennadeployed
    :field nexteepromerrors: cubesat_frame.metadata.nexteepromerrors
    :field last_failed_task_id: cubesat_frame.metadata.last_failed_task_id
    :field messaging_enabled: cubesat_frame.metadata.messaging_enabled
    :field strfwd0_id: cubesat_frame.metadata.strfwd0_id
    :field strfwd1_key: cubesat_frame.metadata.strfwd1_key
    :field strfwd2_value: cubesat_frame.metadata.strfwd2_value
    :field strfwd3_num_tcmds: cubesat_frame.metadata.strfwd3_num_tcmds
    :field npayloadfails: cubesat_frame.metadata.npayloadfails
    :field last_reset_cause: cubesat_frame.metadata.last_reset_cause
    :field bate_battery: cubesat_frame.metadata.bate_battery
    :field mote_transponder: cubesat_frame.metadata.mote_transponder
    :field minicpu: cubesat_frame.metadata.minicpu
    :field minipl: cubesat_frame.metadata.minipl
    :field maxibat: cubesat_frame.metadata.maxibat
    :field maxicpu: cubesat_frame.metadata.maxicpu
    :field ibat_tx_off_charging: cubesat_frame.metadata.ibat_tx_off_charging
    :field ibat_tx_off_discharging: cubesat_frame.metadata.ibat_tx_off_discharging
    :field ibat_tx_low_power_charging: cubesat_frame.metadata.ibat_tx_low_power_charging
    :field ibat_tx_low_power_discharging: cubesat_frame.metadata.ibat_tx_low_power_discharging
    :field ibat_tx_high_power_charging: cubesat_frame.metadata.ibat_tx_high_power_charging
    :field ibat_tx_high_power_discharging: cubesat_frame.metadata.ibat_tx_high_power_discharging
    :field minvbus1: cubesat_frame.metadata.minvbus1
    :field minvbat1: cubesat_frame.metadata.minvbat1
    :field minvcpu: cubesat_frame.metadata.minvcpu
    :field minvbus2: cubesat_frame.metadata.minvbus2
    :field minvbus3: cubesat_frame.metadata.minvbus3
    :field minvbat2: cubesat_frame.metadata.minvbat2
    :field minibat: cubesat_frame.metadata.minibat
    :field maxvbus1: cubesat_frame.metadata.maxvbus1
    :field maxvbat1: cubesat_frame.metadata.maxvbat1
    :field maxvcpu: cubesat_frame.metadata.maxvcpu
    :field maxvbus2: cubesat_frame.metadata.maxvbus2
    :field maxvbus3: cubesat_frame.metadata.maxvbus3
    :field maxvbat2: cubesat_frame.metadata.maxvbat2
    :field maxipl: cubesat_frame.metadata.maxipl
    :field mintpa: cubesat_frame.metadata.mintpa
    :field mintpb: cubesat_frame.metadata.mintpb
    :field mintpc: cubesat_frame.metadata.mintpc
    :field mintpd: cubesat_frame.metadata.mintpd
    :field minttx: cubesat_frame.metadata.minttx
    :field minttx2: cubesat_frame.metadata.minttx2
    :field mintrx: cubesat_frame.metadata.mintrx
    :field mintcpu: cubesat_frame.metadata.mintcpu
    :field maxtpa: cubesat_frame.metadata.maxtpa
    :field maxtpb: cubesat_frame.metadata.maxtpb
    :field maxtpc: cubesat_frame.metadata.maxtpc
    :field maxtpd: cubesat_frame.metadata.maxtpd
    :field maxteps: cubesat_frame.metadata.maxteps
    :field maxttx: cubesat_frame.metadata.maxttx
    :field maxttx2: cubesat_frame.metadata.maxttx2
    :field maxtrx: cubesat_frame.metadata.maxtrx
    :field maxtcpu: cubesat_frame.metadata.maxtcpu
    :field vloc: cubesat_frame.metadata.vloc
    :field v1: cubesat_frame.metadata.v1
    :field i1: cubesat_frame.metadata.i1
    :field i1pk: cubesat_frame.metadata.i1pk
    :field r1: cubesat_frame.metadata.r1
    :field v2oc: cubesat_frame.metadata.v2oc
    :field v2: cubesat_frame.metadata.v2
    :field r2: cubesat_frame.metadata.r2
    :field t0: cubesat_frame.metadata.t0
    :field td: cubesat_frame.metadata.td
    :field state_begin: cubesat_frame.metadata.state_begin
    :field state_end: cubesat_frame.metadata.state_end
    :field state_now: cubesat_frame.metadata.state_now
    :field enable: cubesat_frame.metadata.enable
    :field counter: cubesat_frame.metadata.counter
    :field tmp: cubesat_frame.metadata.tmp
    :field v0: cubesat_frame.metadata.v0
    :field i0: cubesat_frame.metadata.i0
    :field p0: cubesat_frame.metadata.p0
    :field vp0: cubesat_frame.metadata.vp0
    :field ip0: cubesat_frame.metadata.ip0
    :field pp0: cubesat_frame.metadata.pp0
    :field p1: cubesat_frame.metadata.p1
    :field vp1: cubesat_frame.metadata.vp1
    :field ip1: cubesat_frame.metadata.ip1
    :field pp1: cubesat_frame.metadata.pp1
    :field i2: cubesat_frame.metadata.i2
    :field p2: cubesat_frame.metadata.p2
    :field vp2: cubesat_frame.metadata.vp2
    :field ip2: cubesat_frame.metadata.ip2
    :field pp2: cubesat_frame.metadata.pp2
    :field v3: cubesat_frame.metadata.v3
    :field i3: cubesat_frame.metadata.i3
    :field p3: cubesat_frame.metadata.p3
    :field vp3: cubesat_frame.metadata.vp3
    :field ip3: cubesat_frame.metadata.ip3
    :field pp3: cubesat_frame.metadata.pp3
    :field v4: cubesat_frame.metadata.v4
    :field i4: cubesat_frame.metadata.i4
    :field p4: cubesat_frame.metadata.p4
    :field vp4: cubesat_frame.metadata.vp4
    :field ip4: cubesat_frame.metadata.ip4
    :field pp4: cubesat_frame.metadata.pp4
    :field v5: cubesat_frame.metadata.v5
    :field i5: cubesat_frame.metadata.i5
    :field p5: cubesat_frame.metadata.p5
    :field vp5: cubesat_frame.metadata.vp5
    :field ip5: cubesat_frame.metadata.ip5
    :field pp5: cubesat_frame.metadata.pp5
    :field v6: cubesat_frame.metadata.v6
    :field i6: cubesat_frame.metadata.i6
    :field p6: cubesat_frame.metadata.p6
    :field vp6: cubesat_frame.metadata.vp6
    :field ip6: cubesat_frame.metadata.ip6
    :field pp6: cubesat_frame.metadata.pp6
    :field v7: cubesat_frame.metadata.v7
    :field i7: cubesat_frame.metadata.i7
    :field p7: cubesat_frame.metadata.p7
    :field vp7: cubesat_frame.metadata.vp7
    :field ip7: cubesat_frame.metadata.ip7
    :field pp7: cubesat_frame.metadata.pp7
    :field v8: cubesat_frame.metadata.v8
    :field i8: cubesat_frame.metadata.i8
    :field p8: cubesat_frame.metadata.p8
    :field vp8: cubesat_frame.metadata.vp8
    :field ip8: cubesat_frame.metadata.ip8
    :field pp8: cubesat_frame.metadata.pp8
    :field v9: cubesat_frame.metadata.v9
    :field i9: cubesat_frame.metadata.i9
    :field p9: cubesat_frame.metadata.p9
    :field vp9: cubesat_frame.metadata.vp9
    :field ip9: cubesat_frame.metadata.ip9
    :field pp9: cubesat_frame.metadata.pp9
    :field experiment_clock: cubesat_frame.metadata.experiment_clock
    :field experiment_id: cubesat_frame.metadata.experiment_id
    :field frame_number: cubesat_frame.metadata.frame_number
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.cubesat_frame = Hadesicm.CubesatFrame(self._io, self, self._root)

    class IcmgamemsgFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock = self._io.read_u4le()
            self.msg_num = self._io.read_u1()
            self.message = (self._io.read_bytes_full()).decode(u"UTF-8")

        @property
        def days(self):
            if hasattr(self, '_m_days'):
                return self._m_days

            self._m_days = self.sclock // 86400
            return getattr(self, '_m_days', None)

        @property
        def hours(self):
            if hasattr(self, '_m_hours'):
                return self._m_hours

            self._m_hours = (self.sclock % 86400) // 3600
            return getattr(self, '_m_hours', None)

        @property
        def minutes(self):
            if hasattr(self, '_m_minutes'):
                return self._m_minutes

            self._m_minutes = (self.sclock % 3600) // 60
            return getattr(self, '_m_minutes', None)

        @property
        def seconds(self):
            if hasattr(self, '_m_seconds'):
                return self._m_seconds

            self._m_seconds = (self.sclock % 60)
            return getattr(self, '_m_seconds', None)


    class TimeseriesFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.no_value = self._io.read_bytes(0)


    class EphemerisFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.time_utc = self._io.read_u4le()
            self.adr = self._io.read_u2le()
            self.ful = self._io.read_u4le()
            self.fdl = self._io.read_u4le()
            self.tle_epoch = self._io.read_u4le()
            self.tle_xndt2o = self._io.read_u4le()
            self.tle_xndd6o = self._io.read_u4le()
            self.tle_bstar = self._io.read_u4le()
            self.tle_xincl = self._io.read_u4le()
            self.tle_xnodeo = self._io.read_u4le()
            self.tle_eo = self._io.read_u4le()
            self.tle_omegao = self._io.read_u4le()
            self.tle_xmo = self._io.read_u4le()
            self.tle_xno = self._io.read_u4le()
            self.lat = self._io.read_u2le()
            self.lon = self._io.read_u2le()
            self.alt = self._io.read_u2le()
            self.cnt = self._io.read_u1()


    class PowerFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock = self._io.read_u4le()
            self.spa_tmp = self._io.read_u1()
            self.spb_tmp = self._io.read_u1()
            self.spc_tmp = self._io.read_u1()
            self.spd_tmp = self._io.read_u1()
            self.spi_tmp = self._io.read_u2le()
            self.vbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.vbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.vcpuadc_lo_vbus2_tmp = self._io.read_u2le()
            self.vbus3_vbat2_hi_tmp = self._io.read_u2le()
            self.vbat2_lo_ibat_hi_tmp = self._io.read_u2le()
            self.ibat_lo_icpu_hi_tmp = self._io.read_u2le()
            self.icpu_lo_ipl_tmp = self._io.read_u2le()
            self.peaksignal = self._io.read_u1()
            self.modasignal = self._io.read_u1()
            self.lastcmdsignal = self._io.read_u1()
            self.lastcmdnoise = self._io.read_u1()

        @property
        def ibat(self):
            if hasattr(self, '_m_ibat'):
                return self._m_ibat

            self._m_ibat = (((((self.vbat2_lo_ibat_hi_tmp << 8) & 3840) | ((self.ibat_lo_icpu_hi_tmp >> 8) & 4095)) - 4096) if (((self.vbat2_lo_ibat_hi_tmp << 8) & 3840) | ((self.ibat_lo_icpu_hi_tmp >> 8) & 4095)) > 2047 else (((self.vbat2_lo_ibat_hi_tmp << 8) & 3840) | ((self.ibat_lo_icpu_hi_tmp >> 8) & 4095)))
            return getattr(self, '_m_ibat', None)

        @property
        def spb(self):
            if hasattr(self, '_m_spb'):
                return self._m_spb

            self._m_spb = (self.spb_tmp << 1)
            return getattr(self, '_m_spb', None)

        @property
        def seconds(self):
            if hasattr(self, '_m_seconds'):
                return self._m_seconds

            self._m_seconds = (self.sclock % 60)
            return getattr(self, '_m_seconds', None)

        @property
        def spa(self):
            if hasattr(self, '_m_spa'):
                return self._m_spa

            self._m_spa = (self.spa_tmp << 1)
            return getattr(self, '_m_spa', None)

        @property
        def spd(self):
            if hasattr(self, '_m_spd'):
                return self._m_spd

            self._m_spd = (self.spd_tmp << 1)
            return getattr(self, '_m_spd', None)

        @property
        def spc(self):
            if hasattr(self, '_m_spc'):
                return self._m_spc

            self._m_spc = (self.spc_tmp << 1)
            return getattr(self, '_m_spc', None)

        @property
        def minutes(self):
            if hasattr(self, '_m_minutes'):
                return self._m_minutes

            self._m_minutes = (self.sclock % 3600) // 60
            return getattr(self, '_m_minutes', None)

        @property
        def icpu(self):
            if hasattr(self, '_m_icpu'):
                return self._m_icpu

            self._m_icpu = (((self.ibat_lo_icpu_hi_tmp << 4) & 4080) | (self.icpu_lo_ipl_tmp >> 12))
            return getattr(self, '_m_icpu', None)

        @property
        def spi(self):
            if hasattr(self, '_m_spi'):
                return self._m_spi

            self._m_spi = (self.spi_tmp << 1)
            return getattr(self, '_m_spi', None)

        @property
        def ipl(self):
            if hasattr(self, '_m_ipl'):
                return self._m_ipl

            self._m_ipl = (self.icpu_lo_ipl_tmp & 4095)
            return getattr(self, '_m_ipl', None)

        @property
        def hours(self):
            if hasattr(self, '_m_hours'):
                return self._m_hours

            self._m_hours = (self.sclock % 86400) // 3600
            return getattr(self, '_m_hours', None)

        @property
        def vbat2(self):
            if hasattr(self, '_m_vbat2'):
                return self._m_vbat2

            self._m_vbat2 = ((((self.vbus3_vbat2_hi_tmp << 8) & 3840) | (self.vbat2_lo_ibat_hi_tmp >> 8)) * 4)
            return getattr(self, '_m_vbat2', None)

        @property
        def vbus3(self):
            if hasattr(self, '_m_vbus3'):
                return self._m_vbus3

            self._m_vbus3 = ((self.vbus3_vbat2_hi_tmp >> 4) * 4)
            return getattr(self, '_m_vbus3', None)

        @property
        def vcpu(self):
            if hasattr(self, '_m_vcpu'):
                return self._m_vcpu

            self._m_vcpu = (1210 * 4096) // (((self.vbatadc_lo_vcpuadc_hi_tmp << 4) & 4080) | (self.vcpuadc_lo_vbus2_tmp >> 12))
            return getattr(self, '_m_vcpu', None)

        @property
        def days(self):
            if hasattr(self, '_m_days'):
                return self._m_days

            self._m_days = self.sclock // 86400
            return getattr(self, '_m_days', None)

        @property
        def vbat1(self):
            if hasattr(self, '_m_vbat1'):
                return self._m_vbat1

            self._m_vbat1 = ((((self.vbatadc_lo_vcpuadc_hi_tmp << 8) & 3840) | ((self.vbatadc_lo_vcpuadc_hi_tmp >> 8) & 4095)) * 1400) // 1000
            return getattr(self, '_m_vbat1', None)

        @property
        def vbus2(self):
            if hasattr(self, '_m_vbus2'):
                return self._m_vbus2

            self._m_vbus2 = ((self.vcpuadc_lo_vbus2_tmp & 4095) * 4)
            return getattr(self, '_m_vbus2', None)

        @property
        def vbus1_vbat1(self):
            if hasattr(self, '_m_vbus1_vbat1'):
                return self._m_vbus1_vbat1

            self._m_vbus1_vbat1 = (self.vbus1 - self.vbat1)
            return getattr(self, '_m_vbus1_vbat1', None)

        @property
        def vbus3_vbus2(self):
            if hasattr(self, '_m_vbus3_vbus2'):
                return self._m_vbus3_vbus2

            self._m_vbus3_vbus2 = (self.vbus3 - self.vbus2)
            return getattr(self, '_m_vbus3_vbus2', None)

        @property
        def vbus1(self):
            if hasattr(self, '_m_vbus1'):
                return self._m_vbus1

            self._m_vbus1 = ((self.vbusadc_vbatadc_hi_tmp >> 4) * 1400) // 1000
            return getattr(self, '_m_vbus1', None)


    class SunsensorsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.no_value = self._io.read_bytes(0)


    class StatusFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock = self._io.read_u4le()
            self.uptime = self._io.read_u4le()
            self.nrun = self._io.read_u2le()
            self.npayload = self._io.read_u1()
            self.nwire = self._io.read_u1()
            self.ntransponder = self._io.read_u1()
            self.npayloadfails_lstrst_tmp = self._io.read_u1()
            self.bate_mote_tmp = self._io.read_u1()
            self.ntasksnotexecuted = self._io.read_u1()
            self.antennadeployed = self._io.read_u1()
            self.nexteepromerrors = self._io.read_u1()
            self.last_failed_task_id = self._io.read_u1()
            self.messaging_enabled = self._io.read_u1()
            self.strfwd0_id = self._io.read_u1()
            self.strfwd1_key = self._io.read_u2le()
            self.strfwd2_value = self._io.read_u2le()
            self.strfwd3_num_tcmds = self._io.read_u1()

        @property
        def npayloadfails(self):
            if hasattr(self, '_m_npayloadfails'):
                return self._m_npayloadfails

            self._m_npayloadfails = (self.npayloadfails_lstrst_tmp >> 4)
            return getattr(self, '_m_npayloadfails', None)

        @property
        def seconds(self):
            if hasattr(self, '_m_seconds'):
                return self._m_seconds

            self._m_seconds = (self.sclock % 60)
            return getattr(self, '_m_seconds', None)

        @property
        def minutesup(self):
            if hasattr(self, '_m_minutesup'):
                return self._m_minutesup

            self._m_minutesup = (self.uptime % 3600) // 60
            return getattr(self, '_m_minutesup', None)

        @property
        def minutes(self):
            if hasattr(self, '_m_minutes'):
                return self._m_minutes

            self._m_minutes = (self.sclock % 3600) // 60
            return getattr(self, '_m_minutes', None)

        @property
        def hoursup(self):
            if hasattr(self, '_m_hoursup'):
                return self._m_hoursup

            self._m_hoursup = (self.uptime % 86400) // 3600
            return getattr(self, '_m_hoursup', None)

        @property
        def secondsup(self):
            if hasattr(self, '_m_secondsup'):
                return self._m_secondsup

            self._m_secondsup = (self.uptime % 60)
            return getattr(self, '_m_secondsup', None)

        @property
        def mote_transponder(self):
            if hasattr(self, '_m_mote_transponder'):
                return self._m_mote_transponder

            self._m_mote_transponder = (self.bate_mote_tmp & 15)
            return getattr(self, '_m_mote_transponder', None)

        @property
        def hours(self):
            if hasattr(self, '_m_hours'):
                return self._m_hours

            self._m_hours = (self.sclock % 86400) // 3600
            return getattr(self, '_m_hours', None)

        @property
        def daysup(self):
            if hasattr(self, '_m_daysup'):
                return self._m_daysup

            self._m_daysup = self.uptime // 86400
            return getattr(self, '_m_daysup', None)

        @property
        def bate_battery(self):
            if hasattr(self, '_m_bate_battery'):
                return self._m_bate_battery

            self._m_bate_battery = (self.bate_mote_tmp >> 4)
            return getattr(self, '_m_bate_battery', None)

        @property
        def last_reset_cause(self):
            if hasattr(self, '_m_last_reset_cause'):
                return self._m_last_reset_cause

            self._m_last_reset_cause = (self.npayloadfails_lstrst_tmp & 15)
            return getattr(self, '_m_last_reset_cause', None)

        @property
        def days(self):
            if hasattr(self, '_m_days'):
                return self._m_days

            self._m_days = self.sclock // 86400
            return getattr(self, '_m_days', None)


    class FirstHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.type = self._io.read_bits_int_be(4)
            if not  ((self.type == 1) or (self.type == 2) or (self.type == 3) or (self.type == 4) or (self.type == 5) or (self.type == 6) or (self.type == 7) or (self.type == 8) or (self.type == 9) or (self.type == 10) or (self.type == 11) or (self.type == 12) or (self.type == 14) or (self.type == 15)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.type, self._io, u"/types/first_header/seq/0")
            self.address = self._io.read_bits_int_be(4)
            if not  ((self.address == 2)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.address, self._io, u"/types/first_header/seq/1")


    class NebrijagameFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.no_value = self._io.read_bytes(0)


    class TempstatsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock = self._io.read_u4le()
            self.mintpa_tmp = self._io.read_u1()
            self.mintpb_tmp = self._io.read_u1()
            self.mintpc_tmp = self._io.read_u1()
            self.mintpd_tmp = self._io.read_u1()
            self.mintpe = self._io.read_u1()
            self.minteps_tmp = self._io.read_u1()
            self.minttx_tmp = self._io.read_u1()
            self.minttx2_tmp = self._io.read_u1()
            self.mintrx_tmp = self._io.read_u1()
            self.mintcpu_tmp = self._io.read_u1()
            self.maxtpa_tmp = self._io.read_u1()
            self.maxtpb_tmp = self._io.read_u1()
            self.maxtpc_tmp = self._io.read_u1()
            self.maxtpd_tmp = self._io.read_u1()
            self.maxtpe = self._io.read_u1()
            self.maxteps_tmp = self._io.read_u1()
            self.maxttx_tmp = self._io.read_u1()
            self.maxttx2_tmp = self._io.read_u1()
            self.maxtrx_tmp = self._io.read_u1()
            self.maxtcpu_tmp = self._io.read_u1()

        @property
        def mintpc(self):
            if hasattr(self, '_m_mintpc'):
                return self._m_mintpc

            self._m_mintpc = ((self.mintpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpc', None)

        @property
        def maxttx(self):
            if hasattr(self, '_m_maxttx'):
                return self._m_maxttx

            self._m_maxttx = ((self.maxttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxttx', None)

        @property
        def seconds(self):
            if hasattr(self, '_m_seconds'):
                return self._m_seconds

            self._m_seconds = (self.sclock % 60)
            return getattr(self, '_m_seconds', None)

        @property
        def mintpa(self):
            if hasattr(self, '_m_mintpa'):
                return self._m_mintpa

            self._m_mintpa = ((self.mintpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpa', None)

        @property
        def maxttx2(self):
            if hasattr(self, '_m_maxttx2'):
                return self._m_maxttx2

            self._m_maxttx2 = ((self.maxttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxttx2', None)

        @property
        def maxtpd(self):
            if hasattr(self, '_m_maxtpd'):
                return self._m_maxtpd

            self._m_maxtpd = ((self.maxtpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpd', None)

        @property
        def maxtcpu(self):
            if hasattr(self, '_m_maxtcpu'):
                return self._m_maxtcpu

            self._m_maxtcpu = ((self.maxtcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtcpu', None)

        @property
        def minteps(self):
            if hasattr(self, '_m_minteps'):
                return self._m_minteps

            self._m_minteps = ((self.minteps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_minteps', None)

        @property
        def maxtpc(self):
            if hasattr(self, '_m_maxtpc'):
                return self._m_maxtpc

            self._m_maxtpc = ((self.maxtpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpc', None)

        @property
        def mintcpu(self):
            if hasattr(self, '_m_mintcpu'):
                return self._m_mintcpu

            self._m_mintcpu = ((self.mintcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintcpu', None)

        @property
        def minttx(self):
            if hasattr(self, '_m_minttx'):
                return self._m_minttx

            self._m_minttx = ((self.minttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_minttx', None)

        @property
        def maxteps(self):
            if hasattr(self, '_m_maxteps'):
                return self._m_maxteps

            self._m_maxteps = ((self.maxteps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxteps', None)

        @property
        def maxtpb(self):
            if hasattr(self, '_m_maxtpb'):
                return self._m_maxtpb

            self._m_maxtpb = ((self.maxtpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpb', None)

        @property
        def minutes(self):
            if hasattr(self, '_m_minutes'):
                return self._m_minutes

            self._m_minutes = (self.sclock % 3600) // 60
            return getattr(self, '_m_minutes', None)

        @property
        def mintpd(self):
            if hasattr(self, '_m_mintpd'):
                return self._m_mintpd

            self._m_mintpd = ((self.mintpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpd', None)

        @property
        def hours(self):
            if hasattr(self, '_m_hours'):
                return self._m_hours

            self._m_hours = (self.sclock % 86400) // 3600
            return getattr(self, '_m_hours', None)

        @property
        def mintrx(self):
            if hasattr(self, '_m_mintrx'):
                return self._m_mintrx

            self._m_mintrx = ((self.mintrx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintrx', None)

        @property
        def minttx2(self):
            if hasattr(self, '_m_minttx2'):
                return self._m_minttx2

            self._m_minttx2 = ((self.minttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_minttx2', None)

        @property
        def maxtrx(self):
            if hasattr(self, '_m_maxtrx'):
                return self._m_maxtrx

            self._m_maxtrx = ((self.maxtrx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtrx', None)

        @property
        def mintpb(self):
            if hasattr(self, '_m_mintpb'):
                return self._m_mintpb

            self._m_mintpb = ((self.mintpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpb', None)

        @property
        def maxtpa(self):
            if hasattr(self, '_m_maxtpa'):
                return self._m_maxtpa

            self._m_maxtpa = ((self.maxtpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpa', None)

        @property
        def days(self):
            if hasattr(self, '_m_days'):
                return self._m_days

            self._m_days = self.sclock // 86400
            return getattr(self, '_m_days', None)


    class ExtpowerstatsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.v0 = self._io.read_u2le()
            self.i0 = self._io.read_u2le()
            self.p0 = self._io.read_u2le()
            self.vp0 = self._io.read_u2le()
            self.ip0 = self._io.read_u2le()
            self.pp0 = self._io.read_u2le()
            self.v1 = self._io.read_u2le()
            self.i1 = self._io.read_u2le()
            self.p1 = self._io.read_u2le()
            self.vp1 = self._io.read_u2le()
            self.ip1 = self._io.read_u2le()
            self.pp1 = self._io.read_u2le()
            self.v2 = self._io.read_u2le()
            self.i2 = self._io.read_u2le()
            self.p2 = self._io.read_u2le()
            self.vp2 = self._io.read_u2le()
            self.ip2 = self._io.read_u2le()
            self.pp2 = self._io.read_u2le()
            self.v3 = self._io.read_u2le()
            self.i3 = self._io.read_u2le()
            self.p3 = self._io.read_u2le()
            self.vp3 = self._io.read_u2le()
            self.ip3 = self._io.read_u2le()
            self.pp3 = self._io.read_u2le()
            self.v4 = self._io.read_u2le()
            self.i4 = self._io.read_u2le()
            self.p4 = self._io.read_u2le()
            self.vp4 = self._io.read_u2le()
            self.ip4 = self._io.read_u2le()
            self.pp4 = self._io.read_u2le()
            self.v5 = self._io.read_u2le()
            self.i5 = self._io.read_u2le()
            self.p5 = self._io.read_u2le()
            self.vp5 = self._io.read_u2le()
            self.ip5 = self._io.read_u2le()
            self.pp5 = self._io.read_u2le()
            self.v6 = self._io.read_u2le()
            self.i6 = self._io.read_u2le()
            self.p6 = self._io.read_u2le()
            self.vp6 = self._io.read_u2le()
            self.ip6 = self._io.read_u2le()
            self.pp6 = self._io.read_u2le()
            self.v7 = self._io.read_u2le()
            self.i7 = self._io.read_u2le()
            self.p7 = self._io.read_u2le()
            self.vp7 = self._io.read_u2le()
            self.ip7 = self._io.read_u2le()
            self.pp7 = self._io.read_u2le()
            self.v8 = self._io.read_u2le()
            self.i8 = self._io.read_u2le()
            self.p8 = self._io.read_u2le()
            self.vp8 = self._io.read_u2le()
            self.ip8 = self._io.read_u2le()
            self.pp8 = self._io.read_u2le()
            self.v9 = self._io.read_u2le()
            self.i9 = self._io.read_u2le()
            self.p9 = self._io.read_u2le()
            self.vp9 = self._io.read_u2le()
            self.ip9 = self._io.read_u2le()
            self.pp9 = self._io.read_u2le()


    class PowerstatsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock = self._io.read_u4le()
            self.minvbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.minvbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.minvcpuadc_lo_free_tmp = self._io.read_u1()
            self.minvbus2_tmp = self._io.read_u1()
            self.minvbus3_tmp = self._io.read_u1()
            self.minvbat2_tmp = self._io.read_u1()
            self.minibat_tmp = self._io.read_u1()
            self.minicpu = self._io.read_u1()
            self.minipl = self._io.read_u1()
            self.maxvbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.maxvbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.maxvcpuadc_lo_free_tmp = self._io.read_u1()
            self.maxvbus2_tmp = self._io.read_u1()
            self.maxvbus3_tmp = self._io.read_u1()
            self.maxvbat2_tmp = self._io.read_u1()
            self.maxibat = self._io.read_u1()
            self.maxicpu = self._io.read_u1()
            self.maxipl_tmp = self._io.read_u1()
            self.ibat_tx_off_charging = self._io.read_u1()
            self.ibat_tx_off_discharging = self._io.read_u1()
            self.ibat_tx_low_power_charging = self._io.read_u1()
            self.ibat_tx_low_power_discharging = self._io.read_u1()
            self.ibat_tx_high_power_charging = self._io.read_u1()
            self.ibat_tx_high_power_discharging = self._io.read_u1()

        @property
        def minvbus2(self):
            if hasattr(self, '_m_minvbus2'):
                return self._m_minvbus2

            self._m_minvbus2 = ((self.minvbus2_tmp * 16) * 4)
            return getattr(self, '_m_minvbus2', None)

        @property
        def minvbus3(self):
            if hasattr(self, '_m_minvbus3'):
                return self._m_minvbus3

            self._m_minvbus3 = ((self.minvbus3_tmp * 16) * 4)
            return getattr(self, '_m_minvbus3', None)

        @property
        def seconds(self):
            if hasattr(self, '_m_seconds'):
                return self._m_seconds

            self._m_seconds = (self.sclock % 60)
            return getattr(self, '_m_seconds', None)

        @property
        def maxvbat2(self):
            if hasattr(self, '_m_maxvbat2'):
                return self._m_maxvbat2

            self._m_maxvbat2 = ((self.maxvbat2_tmp * 16) * 4)
            return getattr(self, '_m_maxvbat2', None)

        @property
        def maxvcpu(self):
            if hasattr(self, '_m_maxvcpu'):
                return self._m_maxvcpu

            self._m_maxvcpu = (1210 * 4096) // (((self.maxvbatadc_lo_vcpuadc_hi_tmp << 4) & 4080) | (self.maxvcpuadc_lo_free_tmp >> 4))
            return getattr(self, '_m_maxvcpu', None)

        @property
        def maxvbus1(self):
            if hasattr(self, '_m_maxvbus1'):
                return self._m_maxvbus1

            self._m_maxvbus1 = (1400 * (self.maxvbusadc_vbatadc_hi_tmp >> 4)) // 1000
            return getattr(self, '_m_maxvbus1', None)

        @property
        def maxvbat1(self):
            if hasattr(self, '_m_maxvbat1'):
                return self._m_maxvbat1

            self._m_maxvbat1 = (1400 * (((self.maxvbusadc_vbatadc_hi_tmp << 8) & 3840) | ((self.maxvbatadc_lo_vcpuadc_hi_tmp >> 8) & 255))) // 1000
            return getattr(self, '_m_maxvbat1', None)

        @property
        def maxvbus3(self):
            if hasattr(self, '_m_maxvbus3'):
                return self._m_maxvbus3

            self._m_maxvbus3 = ((self.maxvbus3_tmp * 16) * 4)
            return getattr(self, '_m_maxvbus3', None)

        @property
        def maxipl(self):
            if hasattr(self, '_m_maxipl'):
                return self._m_maxipl

            self._m_maxipl = (self.maxipl_tmp << 2)
            return getattr(self, '_m_maxipl', None)

        @property
        def minutes(self):
            if hasattr(self, '_m_minutes'):
                return self._m_minutes

            self._m_minutes = (self.sclock % 3600) // 60
            return getattr(self, '_m_minutes', None)

        @property
        def maxvbus2(self):
            if hasattr(self, '_m_maxvbus2'):
                return self._m_maxvbus2

            self._m_maxvbus2 = ((self.maxvbus2_tmp * 16) * 4)
            return getattr(self, '_m_maxvbus2', None)

        @property
        def minvbat1(self):
            if hasattr(self, '_m_minvbat1'):
                return self._m_minvbat1

            self._m_minvbat1 = (1400 * (((self.minvbusadc_vbatadc_hi_tmp << 8) & 3840) | ((self.minvbatadc_lo_vcpuadc_hi_tmp >> 8) & 255))) // 1000
            return getattr(self, '_m_minvbat1', None)

        @property
        def hours(self):
            if hasattr(self, '_m_hours'):
                return self._m_hours

            self._m_hours = (self.sclock % 86400) // 3600
            return getattr(self, '_m_hours', None)

        @property
        def minibat(self):
            if hasattr(self, '_m_minibat'):
                return self._m_minibat

            self._m_minibat = (self.minibat_tmp * -1)
            return getattr(self, '_m_minibat', None)

        @property
        def days(self):
            if hasattr(self, '_m_days'):
                return self._m_days

            self._m_days = self.sclock // 86400
            return getattr(self, '_m_days', None)

        @property
        def minvbus1(self):
            if hasattr(self, '_m_minvbus1'):
                return self._m_minvbus1

            self._m_minvbus1 = (1400 * (self.minvbusadc_vbatadc_hi_tmp >> 4)) // 1000
            return getattr(self, '_m_minvbus1', None)

        @property
        def minvcpu(self):
            if hasattr(self, '_m_minvcpu'):
                return self._m_minvcpu

            self._m_minvcpu = (1210 * 4096) // (((self.minvbatadc_lo_vcpuadc_hi_tmp << 4) & 4080) | (self.minvcpuadc_lo_free_tmp >> 4))
            return getattr(self, '_m_minvcpu', None)

        @property
        def minvbat2(self):
            if hasattr(self, '_m_minvbat2'):
                return self._m_minvbat2

            self._m_minvbat2 = ((self.minvbat2_tmp * 16) * 4)
            return getattr(self, '_m_minvbat2', None)


    class FraunhoferexpFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.no_value = self._io.read_bytes(0)


    class TempFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock = self._io.read_u4le()
            self.tpa_tmp = self._io.read_u1()
            self.tpb_tmp = self._io.read_u1()
            self.tpc_tmp = self._io.read_u1()
            self.tpd_tmp = self._io.read_u1()
            self.tpe_tmp = self._io.read_u1()
            self.teps_tmp = self._io.read_u1()
            self.ttx_tmp = self._io.read_u1()
            self.ttx2_tmp = self._io.read_u1()
            self.trx_tmp = self._io.read_u1()
            self.tcpu_tmp = self._io.read_u1()

        @property
        def teps(self):
            if hasattr(self, '_m_teps'):
                return self._m_teps

            self._m_teps = ((self.teps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_teps', None)

        @property
        def seconds(self):
            if hasattr(self, '_m_seconds'):
                return self._m_seconds

            self._m_seconds = (self.sclock % 60)
            return getattr(self, '_m_seconds', None)

        @property
        def tpb(self):
            if hasattr(self, '_m_tpb'):
                return self._m_tpb

            self._m_tpb = ((self.tpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpb', None)

        @property
        def trx(self):
            if hasattr(self, '_m_trx'):
                return self._m_trx

            self._m_trx = ((self.trx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_trx', None)

        @property
        def minutes(self):
            if hasattr(self, '_m_minutes'):
                return self._m_minutes

            self._m_minutes = (self.sclock % 3600) // 60
            return getattr(self, '_m_minutes', None)

        @property
        def hours(self):
            if hasattr(self, '_m_hours'):
                return self._m_hours

            self._m_hours = (self.sclock % 86400) // 3600
            return getattr(self, '_m_hours', None)

        @property
        def ttx(self):
            if hasattr(self, '_m_ttx'):
                return self._m_ttx

            self._m_ttx = ((self.ttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_ttx', None)

        @property
        def tpd(self):
            if hasattr(self, '_m_tpd'):
                return self._m_tpd

            self._m_tpd = ((self.tpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpd', None)

        @property
        def tpa(self):
            if hasattr(self, '_m_tpa'):
                return self._m_tpa

            self._m_tpa = ((self.tpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpa', None)

        @property
        def tcpu(self):
            if hasattr(self, '_m_tcpu'):
                return self._m_tcpu

            self._m_tcpu = ((self.tcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tcpu', None)

        @property
        def days(self):
            if hasattr(self, '_m_days'):
                return self._m_days

            self._m_days = self.sclock // 86400
            return getattr(self, '_m_days', None)

        @property
        def ttx2(self):
            if hasattr(self, '_m_ttx2'):
                return self._m_ttx2

            self._m_ttx2 = ((self.ttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_ttx2', None)

        @property
        def tpc(self):
            if hasattr(self, '_m_tpc'):
                return self._m_tpc

            self._m_tpc = ((self.tpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpc', None)


    class ExperimentFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.experiment_clock = self._io.read_u4le()
            self.experiment_id = self._io.read_u1()
            self.frame_number = self._io.read_u1()
            self.experiment_data = self._io.read_bytes_full()


    class DeployFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vloc = self._io.read_u2le()
            self.v1 = self._io.read_u2le()
            self.i1 = self._io.read_u2le()
            self.i1pk = self._io.read_u2le()
            self.r1 = self._io.read_u2le()
            self.v2oc = self._io.read_u2le()
            self.v2 = self._io.read_u2le()
            self.r2 = self._io.read_u2le()
            self.t0 = self._io.read_u4le()
            self.td = self._io.read_u2le()
            self.state_begin = self._io.read_u1()
            self.state_end = self._io.read_u1()
            self.state_now = self._io.read_u1()
            self.enable = self._io.read_u1()
            self.counter = self._io.read_u1()
            self.tmp = self._io.read_u1()


    class CubesatFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.first_header = Hadesicm.FirstHeader(self._io, self, self._root)
            _on = self.first_header.type
            if _on == 14:
                self.metadata = Hadesicm.TimeseriesFrame(self._io, self, self._root)
            elif _on == 10:
                self.metadata = Hadesicm.NebrijagameFrame(self._io, self, self._root)
            elif _on == 4:
                self.metadata = Hadesicm.PowerstatsFrame(self._io, self, self._root)
            elif _on == 6:
                self.metadata = Hadesicm.SunsensorsFrame(self._io, self, self._root)
            elif _on == 7:
                self.metadata = Hadesicm.IcmgamemsgFrame(self._io, self, self._root)
            elif _on == 1:
                self.metadata = Hadesicm.PowerFrame(self._io, self, self._root)
            elif _on == 11:
                self.metadata = Hadesicm.FraunhoferexpFrame(self._io, self, self._root)
            elif _on == 12:
                self.metadata = Hadesicm.EphemerisFrame(self._io, self, self._root)
            elif _on == 3:
                self.metadata = Hadesicm.StatusFrame(self._io, self, self._root)
            elif _on == 5:
                self.metadata = Hadesicm.TempstatsFrame(self._io, self, self._root)
            elif _on == 15:
                self.metadata = Hadesicm.ExperimentFrame(self._io, self, self._root)
            elif _on == 8:
                self.metadata = Hadesicm.DeployFrame(self._io, self, self._root)
            elif _on == 9:
                self.metadata = Hadesicm.ExtpowerstatsFrame(self._io, self, self._root)
            elif _on == 2:
                self.metadata = Hadesicm.TempFrame(self._io, self, self._root)



